@@ -1141,7 +1141,7 @@ void qtractorAudioEngine::createMetroBus (void)
 	// Whether metronome bus is here owned, or...
 	if (m_bMetroBus) {
 		m_pMetroBus = new qtractorAudioBus(this,
-			"Metronome", qtractorBus::Output, 2);
+			"Metronome", qtractorBus::Output);
 	} else {
 		// Metronome bus gets to be the first available output bus...
 		for (qtractorBus *pBus = qtractorEngine::buses().first();
@@ -1167,8 +1167,10 @@ bool qtractorAudioEngine::openMetroBus (void)
 		return false;
 
 	// This is it, when dedicated...
-	if (m_bMetroBus)
+	if (m_bMetroBus) {
+		addBusEx(m_pMetroBus);
 		m_pMetroBus->open();
+	}
 
 	// Enough number of channels?...
 	unsigned short iChannels = m_pMetroBus->channels();
@@ -1200,8 +1202,10 @@ void qtractorAudioEngine::closeMetroBus (void)
 		m_pMetroBeatBuff = NULL;
 	}
 
-	if (m_pMetroBus && m_bMetroBus)
+	if (m_pMetroBus && m_bMetroBus) {
+		removeBusEx(m_pMetroBus);
 		m_pMetroBus->close();
+	}
 
 	m_iMetroBeatStart = 0;
 	m_iMetroBeat = 0;
@@ -1239,11 +1243,18 @@ void qtractorAudioEngine::resetMetro (void)
 	qtractorTimeScale::Cursor& cursor = pSession->timeScale()->cursor();
 	qtractorTimeScale::Node *pNode = cursor.seekFrame(iFrame);
 
-	m_iMetroBeat = pNode->beatFromFrame(iFrame) + 1;
-	m_iMetroBeatStart = pNode->frameFromBeat(m_iMetroBeat);
-
 	// FIXME: Each sample buffer must be bounded properly...
-	unsigned long iMaxLength = (m_iMetroBeatStart / m_iMetroBeat);
+	unsigned long  iMaxLength = 0;
+	unsigned short iNextBeat = pNode->beatFromFrame(iFrame);
+	if (iNextBeat > 0) {
+		m_iMetroBeat = iNextBeat;
+		m_iMetroBeatStart = pNode->frameFromBeat(m_iMetroBeat);
+		iMaxLength = (m_iMetroBeatStart / m_iMetroBeat);
+	} else {
+		m_iMetroBeat = 0;
+		m_iMetroBeatStart = 0;
+		iMaxLength = pNode->frameFromBeat(1);
+	}
 
 	if (m_pMetroBarBuff) {
 		unsigned long iMetroBarLength = m_pMetroBarBuff->frames();
@@ -1299,7 +1310,7 @@ void qtractorAudioEngine::createPlayerBus (void)
 	// Whether audition/pre-listening bus is here owned, or...
 	if (m_bPlayerBus) {
 		m_pPlayerBus = new qtractorAudioBus(this,
-			"Player", qtractorBus::Output, 2);
+			"Player", qtractorBus::Output);
 	} else {
 		// Audition/pre-listening bus gets to be
 		// the first available output bus...
@@ -1325,8 +1336,10 @@ bool qtractorAudioEngine::openPlayerBus (void)
 	if (m_pPlayerBus == NULL)
 		return false;
 
-	if (m_bPlayerBus)
+	if (m_bPlayerBus) {
+		addBusEx(m_pPlayerBus);
 		m_pPlayerBus->open();
+	}
 
 	// Enough number of channels?...
 	unsigned short iChannels = m_pPlayerBus->channels();
@@ -1349,8 +1362,10 @@ void qtractorAudioEngine::closePlayerBus (void)
 		m_pPlayerBuff = NULL;
 	}
 
-	if (m_pPlayerBus && m_bPlayerBus)
+	if (m_pPlayerBus && m_bPlayerBus) {
+		removeBusEx(m_pPlayerBus);
 		m_pPlayerBus->close();
+	}
 }
 
 
@@ -1432,27 +1447,8 @@ void qtractorAudioEngine::closePlayer (void)
 // return the total number of effective (re)connection attempts...
 int qtractorAudioEngine::updateConnects (void)
 {
-	// It must be activated, sure...
-	if (!isActivated())
-		return 0;
-
-	// Do it first on all standard owned dependable buses...
-	int iUpdate = qtractorEngine::updateConnects();
-
-	// Metronome bus outputs...
-	if (m_bMetroBus && m_pMetroBus) {
-		iUpdate += m_pMetroBus->updateConnects(
-			qtractorBus::Output, m_pMetroBus->outputs(), true);
-	}
-
-	// Player bus outputs...
-	if (m_bPlayerBus && m_pPlayerBus) {
-		iUpdate += m_pPlayerBus->updateConnects(
-			qtractorBus::Output, m_pPlayerBus->outputs(), true);
-	}
-
-	// Done.
-	return iUpdate;
+	// Do it as usual, on all standard owned dependable buses...
+	return qtractorEngine::updateConnects();
 }
 
 
@@ -1470,7 +1466,7 @@ qtractorAudioBus::qtractorAudioBus ( qtractorAudioEngine *pAudioEngine,
 
 	if (busMode & qtractorBus::Input) {
 		m_pIAudioMonitor = new qtractorAudioMonitor(iChannels);
-		m_pIPluginList   = createPluginList();
+		m_pIPluginList   = createPluginList(qtractorPluginList::AudioInBus);
 	} else {
 		m_pIAudioMonitor = NULL;
 		m_pIPluginList   = NULL;
@@ -1478,7 +1474,7 @@ qtractorAudioBus::qtractorAudioBus ( qtractorAudioEngine *pAudioEngine,
 
 	if (busMode & qtractorBus::Output) {
 		m_pOAudioMonitor = new qtractorAudioMonitor(iChannels);
-		m_pOPluginList   = createPluginList();
+		m_pOPluginList   = createPluginList(qtractorPluginList::AudioOutBus);
 	} else {
 		m_pOAudioMonitor = NULL;
 		m_pOPluginList   = NULL;
@@ -1538,8 +1534,10 @@ void qtractorAudioBus::setChannels ( unsigned short iChannels )
 	if (m_pOAudioMonitor)
 		m_pOAudioMonitor->setChannels(iChannels);
 
-	updatePluginList(m_pIPluginList);
-	updatePluginList(m_pOPluginList);
+	if (m_pIPluginList)
+		updatePluginList(m_pIPluginList, qtractorPluginList::AudioInBus);
+	if (m_pOPluginList)
+		updatePluginList(m_pOPluginList, qtractorPluginList::AudioOutBus);
 }
 
 unsigned short qtractorAudioBus::channels (void) const
@@ -1614,14 +1612,10 @@ bool qtractorAudioBus::open (void)
 	}
 
 	// Plugin lists need some buffer (re)allocation too...
-	if (m_pIPluginList) {
-		updatePluginList(m_pIPluginList);
-		m_pIPluginList->setName(QObject::tr("%1 In").arg(busName()));
-	}
-	if (m_pOPluginList) {
-		updatePluginList(m_pOPluginList);
-		m_pOPluginList->setName(QObject::tr("%1 Out").arg(busName()));
-	}
+	if (m_pIPluginList)
+		updatePluginList(m_pIPluginList, qtractorPluginList::AudioInBus);
+	if (m_pOPluginList)
+		updatePluginList(m_pOPluginList, qtractorPluginList::AudioOutBus);
 
 	// Finally, open for biz...
 	m_bEnabled = true;
@@ -1764,7 +1758,7 @@ void qtractorAudioBus::updateBusMode (void)
 		if (m_pIAudioMonitor == NULL)
 			m_pIAudioMonitor = new qtractorAudioMonitor(m_iChannels);
 		if (m_pIPluginList == NULL)
-			m_pIPluginList = createPluginList();
+			m_pIPluginList = createPluginList(qtractorPluginList::AudioInBus);
 	} else {
 		if (m_pIAudioMonitor) {
 			delete m_pIAudioMonitor;
@@ -1781,7 +1775,7 @@ void qtractorAudioBus::updateBusMode (void)
 		if (m_pOAudioMonitor == NULL)
 			m_pOAudioMonitor = new qtractorAudioMonitor(m_iChannels);
 		if (m_pOPluginList == NULL)
-			m_pOPluginList = createPluginList();
+			m_pOPluginList = createPluginList(qtractorPluginList::AudioOutBus);
 	} else {
 		if (m_pOAudioMonitor) {
 			delete m_pOAudioMonitor;
@@ -1964,7 +1958,7 @@ qtractorPluginList *qtractorAudioBus::pluginList_out (void) const
 
 
 // Create plugin-list properly.
-qtractorPluginList *qtractorAudioBus::createPluginList (void) const
+qtractorPluginList *qtractorAudioBus::createPluginList ( int iFlags ) const
 {
 	qtractorAudioEngine *pAudioEngine
 		= static_cast<qtractorAudioEngine *> (engine());
@@ -1972,28 +1966,40 @@ qtractorPluginList *qtractorAudioBus::createPluginList (void) const
 		return NULL;
 
 	// Create plugin-list alright...
-	return new qtractorPluginList(m_iChannels,
-		pAudioEngine->bufferSize(), pAudioEngine->sampleRate(),
-		qtractorPluginList::AudioBus);
+	qtractorPluginList *pPluginList = new qtractorPluginList(m_iChannels,
+		pAudioEngine->bufferSize(), pAudioEngine->sampleRate(), iFlags);
+
+	// Set plugin-list title name...
+	updatePluginListName(pPluginList, iFlags);
+
+	return pPluginList;
 }
 
 
-// Update plugin-list buffers properly.
-void qtractorAudioBus::updatePluginList ( qtractorPluginList *pPluginList )
+// Update plugin-list title name...
+void qtractorAudioBus::updatePluginListName (
+	qtractorPluginList *pPluginList, int iFlags ) const
+{
+	pPluginList->setName((iFlags & qtractorPluginList::In ?
+		QObject::tr("%1 In") : QObject::tr("%1 Out")).arg(busName()));
+}
+
+// Update plugin-list name/buffers properly.
+void qtractorAudioBus::updatePluginList (
+	qtractorPluginList *pPluginList, int iFlags )
 {
 	// Sanity checks...
-	if (pPluginList == NULL)
-		return;
-
 	qtractorAudioEngine *pAudioEngine
 		= static_cast<qtractorAudioEngine *> (engine());
 	if (pAudioEngine == NULL)
 		return;
-	
+
+	// Set plugin-list title name...
+	updatePluginListName(pPluginList, iFlags);
+
 	// Set plugin-list buffer alright...
 	pPluginList->setBuffer(m_iChannels,
-		pAudioEngine->bufferSize(), pAudioEngine->sampleRate(),
-		qtractorPluginList::AudioBus);
+		pAudioEngine->bufferSize(), pAudioEngine->sampleRate(), iFlags);
 }
 
 
@@ -2032,7 +2038,8 @@ int qtractorAudioBus::updateConnects ( qtractorBus::BusMode busMode,
 			int iClientPort = 0;
 			while (ppszClientPorts[iClientPort]) {
 				// Check if already in list/connected...
-				const QString sClientPort = ppszClientPorts[iClientPort];
+				const QString sClientPort
+					= QString::fromUtf8(ppszClientPorts[iClientPort]);
 				item.clientName = sClientPort.section(':', 0, 0);
 				item.portName   = sClientPort.section(':', 1, 1);
 				ConnectItem *pItem = connects.findItem(item);
