@@ -277,10 +277,15 @@ protected:
 	void updateBusMode();
 
 	// Create plugin-list properly.
-	qtractorPluginList *createPluginList (void) const;
+	qtractorPluginList *createPluginList(int iFlags) const;
+
+	// Set plugin-list title name.
+	void updatePluginListName(
+		qtractorPluginList *pPluginList, int iFlags) const;
 
 	// Set plugin-list buffers properly.
-	void updatePluginList(qtractorPluginList *pPluginList);
+	void updatePluginList(
+		qtractorPluginList *pPluginList, int iFlags);
 
 private:
 
