@@ -26,7 +26,12 @@
 #include "qtractorEngineCommand.h"
 #include "qtractorAudioEngine.h"
 #include "qtractorMidiEngine.h"
+#include "qtractorMidiBuffer.h"
 
+#include "qtractorMidiSysexForm.h"
+#include "qtractorMidiSysex.h"
+
+#include "qtractorInstrument.h"
 #include "qtractorPlugin.h"
 
 #include <QHeaderView>
@@ -54,15 +59,33 @@ public:
 		switch (m_pBus->busType()) {
 		case qtractorTrack::Audio:
 			QTreeWidgetItem::setIcon(0, QIcon(":/icons/trackAudio.png"));
+			QTreeWidgetItem::setText(1, QString::number(
+				static_cast<qtractorAudioBus *> (m_pBus)->channels()));
 			break;
 		case qtractorTrack::Midi:
 			QTreeWidgetItem::setIcon(0, QIcon(":/icons/trackMidi.png"));
+			QTreeWidgetItem::setText(1, QString::number(16));
 			break;
 		case qtractorTrack::None:
 		default:
 			break;
 		}
 		QTreeWidgetItem::setText(0, m_pBus->busName());
+		switch (m_pBus->busMode()) {
+		case qtractorBus::Duplex:
+			QTreeWidgetItem::setText(2, QObject::tr("Duplex"));
+			break;
+		case qtractorBus::Output:
+			QTreeWidgetItem::setText(2, QObject::tr("Output"));
+			break;
+		case qtractorBus::Input:
+			QTreeWidgetItem::setText(2, QObject::tr("Input"));
+			break;
+		case qtractorBus::None:
+		default:
+			QTreeWidgetItem::setText(2, QObject::tr("None"));
+			break;
+		}
 	}
 
 	// Bus accessors.
@@ -95,7 +118,10 @@ qtractorBusForm::qtractorBusForm (
 	m_iDirtyTotal = 0;
 
 	QHeaderView *pHeader = m_ui.BusListView->header();
-	pHeader->setResizeMode(QHeaderView::Custom);
+//	pHeader->setResizeMode(QHeaderView::Custom);
+	pHeader->resizeSection(0, 160);
+ 	pHeader->setResizeMode(1, QHeaderView::ResizeToContents);
+ 	pHeader->setResizeMode(2, QHeaderView::ResizeToContents);
 	pHeader->setDefaultAlignment(Qt::AlignLeft);
 	pHeader->setMovable(false);
 
@@ -133,6 +159,12 @@ qtractorBusForm::qtractorBusForm (
 	QObject::connect(m_ui.AudioAutoConnectCheckBox,
 		SIGNAL(clicked()),
 		SLOT(changed()));
+	QObject::connect(m_ui.MidiInstrumentComboBox,
+		SIGNAL(activated(int)),
+		SLOT(changed()));
+	QObject::connect(m_ui.MidiSysexPushButton,
+		SIGNAL(clicked()),
+		SLOT(midiSysex()));
 
 	QObject::connect(m_ui.InputPluginListView,
 		SIGNAL(currentRowChanged(int)),
@@ -244,12 +276,15 @@ void qtractorBusForm::showBus ( qtractorBus *pBus )
 	m_iDirtySetup++;
 
 	// Reset plugin lists...
-	m_ui.InputPluginListView->setPluginList(NULL);
-	m_ui.OutputPluginListView->setPluginList(NULL);
+	resetPluginLists();
 
 	// Settle current bus reference...
 	m_pBus = pBus;
 
+	// Update some dependable specifics...
+	updateMidiInstruments();
+	updateMidiSysex();
+
 	// Show bus properties into view pane...
 	if (pBus) {
 		QString sBusTitle = pBus->busName();
@@ -262,6 +297,7 @@ void qtractorBusForm::showBus ( qtractorBus *pBus )
 			qtractorAudioBus *pAudioBus
 				= static_cast<qtractorAudioBus *> (pBus);
 			if (pAudioBus) {
+				// Audio bus specifics...
 				m_ui.AudioChannelsSpinBox->setValue(
 					pAudioBus->channels());
 				m_ui.AudioAutoConnectCheckBox->setChecked(
@@ -282,6 +318,12 @@ void qtractorBusForm::showBus ( qtractorBus *pBus )
 			qtractorMidiBus *pMidiBus
 				= static_cast<qtractorMidiBus *> (pBus);
 			if (pMidiBus) {
+				// MIDI bus specifics...
+				int iInstrumentIndex
+					= m_ui.MidiInstrumentComboBox->findText(
+						pMidiBus->instrumentName());
+				m_ui.MidiInstrumentComboBox->setCurrentIndex(
+					iInstrumentIndex > 0 ? iInstrumentIndex : 0);
 				// Set plugin lists...
 				if (pMidiBus->busMode() & qtractorBus::Input)
 					m_ui.InputPluginListView->setPluginList(
@@ -396,7 +438,7 @@ void qtractorBusForm::selectBus (void)
 			}
 			// Fall thru...
 		case QMessageBox::Discard:
-			break;;
+			break;
 		default:    // Cancel.
 			return;
 		}
@@ -407,21 +449,27 @@ void qtractorBusForm::selectBus (void)
 }
 
 
-// Check whether the current view is elligible as a new bus.
-bool qtractorBusForm::canCreateBus (void) const
+// Check whether the current view is elligible for action.
+unsigned int qtractorBusForm::flags (void) const
 {
-	if (m_iDirtyCount == 0)
-		return false;
-	if (m_pBus == NULL)
-		return false;
+	unsigned int iFlags = 0;
 
 	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
-		return false;
+		return iFlags;
+
+	if (m_pBus == NULL)
+		return iFlags;
+
+	if (m_pBus->prev())
+		iFlags |= Delete;
+	
+	if (m_iDirtyCount == 0)
+		return iFlags;
 
 	const QString sBusName = m_ui.BusNameLineEdit->text().simplified();
 	if (sBusName.isEmpty())
-		return false;
+		return iFlags;
 
 	// Get the device view root item...
 	qtractorEngine *pEngine = NULL;
@@ -437,51 +485,22 @@ bool qtractorBusForm::canCreateBus (void) const
 	}
 	// Is it still valid?
 	if (pEngine == NULL)
-		return false;
+		return iFlags;
 
 	// Is there one already?
-	return (pEngine->findBus(sBusName) == NULL);
-}
+	qtractorBus *pBus = pEngine->findBus(sBusName);
+	if (pBus == NULL)
+		iFlags |= Create;
+	if ((pBus == NULL || pBus == m_pBus)
+		&& (m_pBus->prev() || m_ui.BusModeComboBox->currentIndex() == 2))
+		iFlags |= Update;
 
-
-// Check whether the current view is elligible for update.
-bool qtractorBusForm::canUpdateBus (void) const
-{
-	if (m_iDirtyCount == 0)
-		return false;
-	if (m_pBus == NULL)
-		return false;
-
-	qtractorSession *pSession = qtractorSession::getInstance();
-	if (pSession == NULL)
-		return false;
-
-	const QString sBusName = m_ui.BusNameLineEdit->text().simplified();
-	if (sBusName.isEmpty())
-		return false;
-
-	// Master (default) buses must be duplex...
-	return (m_pBus->prev() || m_ui.BusModeComboBox->currentIndex() == 2);
-}
-
-
-// Check whether the current view is elligible for deletion.
-bool qtractorBusForm::canDeleteBus (void) const
-{
-	if (m_pBus == NULL)
-		return false;
-
-	qtractorSession *pSession = qtractorSession::getInstance();
-	if (pSession == NULL)
-		return false;
-
-	// The very first bus is never deletable...
-	return (m_pBus->prev() != NULL);
+	return iFlags;
 }
 
 
 // Update bus method.
-bool qtractorBusForm::updateBusEx ( qtractorBus *pBus ) const
+bool qtractorBusForm::updateBusEx ( qtractorBus *pBus )
 {
 	if (pBus == NULL)
 		return false;
@@ -494,6 +513,9 @@ bool qtractorBusForm::updateBusEx ( qtractorBus *pBus ) const
 	if (sBusName.isEmpty())
 		return false;
 
+	// Reset plugin lists...
+	resetPluginLists();
+
 	qtractorBus::BusMode busMode = qtractorBus::None;
 	switch (m_ui.BusModeComboBox->currentIndex()) {
 	case 0:
@@ -529,6 +551,11 @@ bool qtractorBusForm::updateBusEx ( qtractorBus *pBus ) const
 			m_ui.AudioAutoConnectCheckBox->isChecked());
 		break;
 	case qtractorTrack::Midi:
+		pUpdateBusCommand->setInstrumentName(
+			m_ui.MidiInstrumentComboBox->currentIndex() > 0
+			? m_ui.MidiInstrumentComboBox->currentText()
+			: QString::null);
+		// Fall thru...
 	case qtractorTrack::None:
 	default:
 		break;
@@ -588,6 +615,11 @@ void qtractorBusForm::createBus (void)
 			m_ui.AudioAutoConnectCheckBox->isChecked());
 		break;
 	case qtractorTrack::Midi:
+		pCreateBusCommand->setInstrumentName(
+			m_ui.MidiInstrumentComboBox->currentIndex() > 0
+			? m_ui.MidiInstrumentComboBox->currentText()
+			: QString::null);
+		// Fall thru...
 	case qtractorTrack::None:
 	default:
 		break;
@@ -655,6 +687,9 @@ void qtractorBusForm::deleteBus (void)
 			return;
 	}
 
+	// Reset plugin lists...
+	resetPluginLists();
+
 	// Make it as an unduable command...
 	qtractorDeleteBusCommand *pDeleteBusCommand
 		= new qtractorDeleteBusCommand(m_pBus);
@@ -673,6 +708,14 @@ void qtractorBusForm::deleteBus (void)
 }
 
 
+// Reset (stabilize) plugin lists...
+void qtractorBusForm::resetPluginLists (void)
+{
+	m_ui.InputPluginListView->setPluginList(NULL);
+	m_ui.OutputPluginListView->setPluginList(NULL);
+}
+
+
 // Make changes due.
 void qtractorBusForm::changed (void)
 {
@@ -714,19 +757,25 @@ void qtractorBusForm::stabilizeForm (void)
 {
 	if (m_pBus) {
 		m_ui.CommonBusGroup->setEnabled(true);
-		m_ui.AudioBusGroup->setEnabled(m_pBus->busType() == qtractorTrack::Audio);
+		m_ui.AudioBusGroup->setEnabled(
+			m_pBus->busType() == qtractorTrack::Audio);
+		m_ui.MidiBusGroup->setEnabled(
+			m_pBus->busType() == qtractorTrack::Midi &&
+			(m_pBus->busMode() & qtractorBus::Output));
 	} else {
 		m_ui.CommonBusGroup->setEnabled(false);
 		m_ui.AudioBusGroup->setEnabled(false);
+		m_ui.MidiBusGroup->setEnabled(false);
 	}
 
 	m_ui.PassthruCheckBox->setEnabled(
 		m_pBus && m_ui.BusModeComboBox->currentIndex() == 2);
 
+	unsigned int iFlags = flags();
 	m_ui.RefreshPushButton->setEnabled(m_iDirtyCount > 0);
-	m_ui.CreatePushButton->setEnabled(canCreateBus());
-	m_ui.UpdatePushButton->setEnabled(canUpdateBus());
-	m_ui.DeletePushButton->setEnabled(canDeleteBus());
+	m_ui.CreatePushButton->setEnabled(iFlags & Create);
+	m_ui.UpdatePushButton->setEnabled(iFlags & Update);
+	m_ui.DeletePushButton->setEnabled(iFlags & Delete);
 
 	// Stabilize current plugin lists state.
 	bool bEnabled;
@@ -782,21 +831,23 @@ void qtractorBusForm::contextMenu ( const QPoint& /*pos*/ )
 	// Build the device context menu...
 	QMenu menu(this);
 	QAction *pAction;
-	
+
+	unsigned int iFlags = flags();
+
 	pAction = menu.addAction(
 		QIcon(":/icons/formCreate.png"),
 		tr("&Create"), this, SLOT(createBus()));
-	pAction->setEnabled(canCreateBus());
+	pAction->setEnabled(iFlags & Create);
 
 	pAction = menu.addAction(
 		QIcon(":/icons/formAccept.png"),
 		tr("&Update"), this, SLOT(updateBus()));
-	pAction->setEnabled(canUpdateBus());
+	pAction->setEnabled(iFlags & Update);
 
 	pAction = menu.addAction(
 		QIcon(":/icons/formRemove.png"),
 		tr("&Delete"), this, SLOT(deleteBus()));
-	pAction->setEnabled(canDeleteBus());
+	pAction->setEnabled(iFlags & Delete);
 
 	menu.addSeparator();
 
@@ -810,6 +861,122 @@ void qtractorBusForm::contextMenu ( const QPoint& /*pos*/ )
 }
 
 
+// Show MIDI SysEx bank manager dialog...
+void qtractorBusForm::midiSysex (void)
+{
+	// Care of MIDI output bus...
+	if (m_pBus == NULL)
+		return;
+	if (m_pBus->busType() != qtractorTrack::Midi)
+		return;
+	if ((m_pBus->busMode() & qtractorBus::Output) == 0)
+		return;
+
+	qtractorMidiBus *pMidiBus
+		= static_cast<qtractorMidiBus *> (m_pBus);
+	if (pMidiBus == NULL)
+		return;
+	if (pMidiBus->sysexList() == NULL)
+		return;
+
+	qtractorMidiSysexForm form(this);
+	form.setSysexList(pMidiBus->sysexList());
+	form.exec();
+
+	updateMidiSysex();
+}
+
+
+// Refresh instrument list.
+void qtractorBusForm::updateMidiInstruments (void)
+{
+	m_ui.MidiInstrumentComboBox->clear();
+	m_ui.MidiInstrumentComboBox->addItem(tr("(No instrument)"));
+
+	// Care of MIDI output bus...
+	if (m_pBus == NULL)
+		return;
+	if (m_pBus->busType() != qtractorTrack::Midi)
+		return;
+	if ((m_pBus->busMode() & qtractorBus::Output) == 0)
+		return;
+
+	qtractorMidiBus *pMidiBus
+		= static_cast<qtractorMidiBus *> (m_pBus);
+	if (pMidiBus == NULL)
+		return;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	qtractorInstrumentList *pInstruments = pSession->instruments();
+	if (pInstruments == NULL)
+		return;
+
+	// Avoid superfluous change notifications...
+	m_iDirtySetup++;
+
+	const QIcon& icon = QIcon(":/icons/itemInstrument.png");
+	if (pMidiBus->pluginList_out()) {
+		qtractorMidiManager *pMidiManager
+			= (pMidiBus->pluginList_out())->midiManager();
+		if (pMidiManager) {
+			pMidiManager->updateInstruments();
+			const qtractorMidiManager::Instruments& list
+				= pMidiManager->instruments();
+			qtractorMidiManager::Instruments::ConstIterator iter = list.constBegin();
+			for ( ; iter != list.constEnd(); ++iter)
+				m_ui.MidiInstrumentComboBox->addItem(icon, iter.key());
+		}
+	}
+
+	// Regular instrument names...
+	qtractorInstrumentList::ConstIterator iter = pInstruments->constBegin();
+	for ( ; iter != pInstruments->constEnd(); ++iter)
+		m_ui.MidiInstrumentComboBox->addItem(icon, iter.value().instrumentName());
+
+	// Done.
+	m_iDirtySetup--;
+}
+
+
+// Update SysEx status.
+void qtractorBusForm::updateMidiSysex (void)
+{
+	m_ui.MidiSysexTextLabel->clear();
+
+	// Care of MIDI output bus...
+	if (m_pBus == NULL)
+		return;
+	if (m_pBus->busType() != qtractorTrack::Midi)
+		return;
+	if ((m_pBus->busMode() & qtractorBus::Output) == 0)
+		return;
+
+	qtractorMidiBus *pMidiBus
+		= static_cast<qtractorMidiBus *> (m_pBus);
+	if (pMidiBus == NULL)
+		return;
+	if (pMidiBus->sysexList() == NULL)
+		return;
+
+	// Show proper count status...
+	int iSysexCount = (pMidiBus->sysexList())->count();
+	switch (iSysexCount) {
+	case 0:
+		m_ui.MidiSysexTextLabel->setText(tr("(none)"));
+		break;
+	case 1:
+		m_ui.MidiSysexTextLabel->setText(tr("(1 item)"));
+		break;
+	default:
+		m_ui.MidiSysexTextLabel->setText(tr("(%1 items)").arg(iSysexCount));
+		break;
+	}
+}
+
+
 // Input plugin list slots.
 void qtractorBusForm::addInputPlugin (void)
 {
