@@ -32,6 +32,8 @@
 #include <QTabWidget>
 #include <QComboBox>
 
+#include <QKeyEvent>
+
 
 //-------------------------------------------------------------------------
 // qtractorConnections - Connections dockable window.
@@ -71,19 +73,16 @@ qtractorConnections::qtractorConnections (
 
 	// Get previously saved splitter sizes,
 	// (with fair default...)
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm) {
-		qtractorOptions *pOptions = pMainForm->options();
-		if (pOptions) {
-			QList<int> sizes;
-			sizes.append(180);
-			sizes.append(60);
-			sizes.append(180);
-			pOptions->loadSplitterSizes(
-				m_pConnectForm->audioConnectSplitter(), sizes);
-			pOptions->loadSplitterSizes(
-				m_pConnectForm->midiConnectSplitter(), sizes);
-		}
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions) {
+		QList<int> sizes;
+		sizes.append(180);
+		sizes.append(60);
+		sizes.append(180);
+		pOptions->loadSplitterSizes(
+			m_pConnectForm->audioConnectSplitter(), sizes);
+		pOptions->loadSplitterSizes(
+			m_pConnectForm->midiConnectSplitter(), sizes);
 	}
 }
 
@@ -91,16 +90,13 @@ qtractorConnections::qtractorConnections (
 // Destructor.
 qtractorConnections::~qtractorConnections (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm) {
-		qtractorOptions *pOptions = pMainForm->options();
-		if (pOptions) {
-			// Get previously saved splitter sizes...
-			pOptions->saveSplitterSizes(
-				m_pConnectForm->audioConnectSplitter());
-			pOptions->saveSplitterSizes(
-				m_pConnectForm->midiConnectSplitter());
-		}
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions) {
+		// Get previously saved splitter sizes...
+		pOptions->saveSplitterSizes(
+			m_pConnectForm->audioConnectSplitter());
+		pOptions->saveSplitterSizes(
+			m_pConnectForm->midiConnectSplitter());
 	}
 
 	// No need to delete child widgets, Qt does it all for us.
@@ -159,19 +155,11 @@ qtractorConnectForm *qtractorConnections::connectForm (void) const
 }
 
 
-// Session accessor.
-qtractorSession *qtractorConnections::session (void) const
-{
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	return (pMainForm ? pMainForm->session() : NULL);
-}
-
-
 // Main bus mode switching.
 void qtractorConnections::showBus ( qtractorBus *pBus,
 	qtractorBus::BusMode busMode )
 {
-	qtractorSession *pSession = session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
@@ -261,4 +249,25 @@ void qtractorConnections::clear (void)
 }
 
 
+// Keyboard event handler.
+void qtractorConnections::keyPressEvent ( QKeyEvent *pKeyEvent )
+{
+#ifdef CONFIG_DEBUG_0
+	qDebug("qtractorConnections::keyPressEvent(%d)", pKeyEvent->key());
+#endif
+	int iKey = pKeyEvent->key();
+	switch (iKey) {
+	case Qt::Key_Escape:
+		close();
+		break;
+	default:
+		QWidget::keyPressEvent(pKeyEvent);
+		break;
+	}
+
+	// Make sure we've get focus back...
+	QWidget::setFocus();
+}
+
+
 // end of qtractorConnections.cpp
