@@ -28,7 +28,7 @@
 
 
 //----------------------------------------------------------------------------
-// qtractorDssiPluginType -- LADSPA plugin type instance.
+// qtractorDssiPluginType -- DSSI plugin type instance.
 //
 
 class qtractorDssiPluginType : public qtractorLadspaPluginType
@@ -99,6 +99,21 @@ public:
 	// The main plugin processing procedure.
 	void process(float **ppIBuffer, float **ppOBuffer, unsigned int nframes);
 
+	// Parameter update method.
+	void updateParam(qtractorPluginParam *pParam, float fValue);
+
+	// Bank/program selector override.
+	void selectProgram(int iBank, int iProg);
+
+	// Provisional program/patch accessor.
+	bool getProgram(int iIndex, Program& program) const;
+
+	// Continuous controller handler.
+	void setController(int iController, int iValue);
+
+	// Configuration (CLOB) stuff.
+	void configure(const QString& sKey, const QString& sValue);
+
 	// GUI Editor stuff.
 	void openEditor(QWidget */*pParent*/);
 	void closeEditor();
@@ -107,30 +122,27 @@ public:
 	void setEditorVisible(bool bVisible);
 	bool isEditorVisible() const;
 
+	// Specific accessors.
+	const DSSI_Descriptor *dssi_descriptor() const;
+
+protected:
+
+	// Post-(re)initializer.
+	void resetChannels();
+
 private:
 
+	// Care of multiple instances here.
+	class DssiMulti *m_pDssiMulti;
+
+	// Controller port map.
+	qtractorPluginParam *m_apControllerMap[128];
+
 	// GUI editor visiability status.
 	bool m_bEditorVisible;
 };
 
 
-//----------------------------------------------------------------------------
-// qtractorDssiPluginParam -- DSSI plugin control input port instance.
-//
-
-class qtractorDssiPluginParam : public qtractorLadspaPluginParam
-{
-public:
-
-	// Constructors.
-	qtractorDssiPluginParam(qtractorDssiPlugin *pDssiPlugin,
-		unsigned long iIndex);
-
-	// Destructor.
-	~qtractorDssiPluginParam();
-};
-
-
 #endif  // __qtractorDssiPlugin_h
 
 // end of qtractorDssiPlugin.h
