@@ -37,6 +37,15 @@
 #define lighter light
 #endif
 
+#if QT_VERSION < 0x040500
+namespace Qt {
+const WindowFlags WindowCloseButtonHint = WindowFlags(0x08000000);
+#if QT_VERSION < 0x040200
+const WindowFlags CustomizeWindowHint   = WindowFlags(0x02000000);
+#endif
+}
+#endif
+
 
 //----------------------------------------------------------------------
 // class qtractorMidiClip -- Audio file/buffer clip.
@@ -187,9 +196,33 @@ bool qtractorMidiClip::openMidiFile ( qtractorMidiFile *pFile,
 		if (!pFile->readTrack(m_pSeq, iTrackChannel))
 			return false;
 		// FIXME: On demand, set session time properties from MIDI file...
-		if (m_bSessionFlag && pFile->tempoMap()) {
-			pFile->tempoMap()->intoTimeScale(pSession->timeScale(), t0);
-			pSession->updateTimeScale();
+		if (m_bSessionFlag) {
+			// Import eventual SysEx setup...
+			// - take care that given track might not be currently open,
+			//   so that we'll resolve MIDI output bus somehow...
+			qtractorMidiBus *pMidiBus = NULL;
+			qtractorMidiEngine *pMidiEngine = pSession->midiEngine();
+			if (pMidiEngine) {
+				pMidiBus = static_cast<qtractorMidiBus *> (
+					pMidiEngine->findBus(pTrack->outputBusName()));
+				if (pMidiBus == NULL) {
+					for (qtractorBus *pBus = pMidiEngine->buses().first();
+							pBus; pBus = pBus->next()) {
+						if (pBus->busMode() & qtractorBus::Output) {
+							pMidiBus = static_cast<qtractorMidiBus *> (pBus);
+							break;
+						}
+					}
+				}
+			}
+			// Import eventual SysEx setup...
+			if (pMidiBus)
+				pMidiBus->importSysexList(m_pSeq);
+			// Import tempo map as well...
+			if (pFile->tempoMap()) {
+				pFile->tempoMap()->intoTimeScale(pSession->timeScale(), t0);
+				pSession->updateTimeScale();
+			}
 			// Reset session flag now.
 			m_bSessionFlag = false;
 		}
@@ -551,15 +584,11 @@ bool qtractorMidiClip::startEditor ( QWidget *pParent )
 		// Build up the editor form...
 		// What style do we create tool childs?
 		Qt::WindowFlags wflags = Qt::Window
-		#if QT_VERSION >= 0x040200
 			| Qt::CustomizeWindowHint
-		#if QT_VERSION >= 0x040500
-			| Qt::WindowCloseButtonHint
-		#endif
-		#endif
 			| Qt::WindowTitleHint
 			| Qt::WindowSystemMenuHint
-			| Qt::WindowMinMaxButtonsHint;
+			| Qt::WindowMinMaxButtonsHint
+			| Qt::WindowCloseButtonHint;
 		qtractorOptions *pOptions = qtractorOptions::getInstance();
 		if (pOptions && pOptions->bKeepToolsOnTop)
 			wflags |= Qt::Tool;
