@@ -1,7 +1,7 @@
 // qtractorMidiEditCommand.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -111,6 +111,36 @@ private:
 };
 
 
+//----------------------------------------------------------------------
+// class qtractorMidiClipCommand - declaration.
+//
+
+class qtractorMidiClipCommand : public qtractorCommand
+{
+public:
+
+	// Constructor.
+	qtractorMidiClipCommand(const QString& sName);
+	// Destructor.
+	virtual ~qtractorMidiClipCommand();
+
+	// Composite command methods.
+	void addEditCommand(qtractorMidiEditCommand *pEditCommand);
+
+	// Composite predicate.
+	bool isEmpty() const;
+
+	// Virtual command methods.
+	bool redo();
+	bool undo();
+
+private:
+
+	// Multi-clip command list.
+	QList<qtractorMidiEditCommand *> m_editCommands;
+};
+
+
 #endif	// __qtractorMidiEditCommand_h
 
 // end of qtractorMidiEditCommand.h
