@@ -1,7 +1,7 @@
 // qtractorMidiEditTime.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -26,9 +26,10 @@
 
 #include "qtractorSessionCommand.h"
 
-#include "qtractorSession.h"
 #include "qtractorMainForm.h"
 
+#include "qtractorTimeScaleForm.h"
+
 #include <QApplication>
 
 #include <QResizeEvent>
@@ -109,30 +110,41 @@ void qtractorMidiEditTime::updatePixmap ( int cx, int /*cy*/)
 	// Account for the editing offset:
 	int dx = cx + pTimeScale->pixelFromFrame(m_pEditor->offset());
 
-	unsigned short iBeat = pTimeScale->beatFromPixel(dx);
-	unsigned long iFrameFromBeat = pTimeScale->frameFromBeat(iBeat);
-	unsigned long iFramesPerBeat = pTimeScale->frameFromBeat(1);
-	unsigned int  iPixelsPerBeat = pTimeScale->pixelFromBeat(1);
-	x = pTimeScale->pixelFromFrame(iFrameFromBeat) - dx;
+	qtractorTimeScale::Cursor cursor(pTimeScale);
+	qtractorTimeScale::Node *pNode = cursor.seekPixel(dx);
+
+	unsigned short iPixelsPerBeat = pNode->pixelsPerBeat();
+	unsigned int iBeat = pNode->beatFromPixel(dx);
+	int x0 = x = pNode->pixelFromBeat(iBeat) - dx;
 	while (x < w) {
-		bool bBeatIsBar = pTimeScale->beatIsBar(iBeat);
+		bool bBeatIsBar = pNode->beatIsBar(iBeat) && (x >= x0);
 		if (bBeatIsBar) {
 			y1 = 0;
 			p.setPen(pal.windowText().color());
-			p.drawText(x + 2, y1 + fm.ascent(), QString::number(
-				1 + (iBeat / pTimeScale->beatsPerBar())));
+			p.drawText(x + 2, y1 + fm.ascent(),
+				QString::number(pNode->barFromBeat(iBeat) + 1));
+			x0 = x + 16;
+			if (iBeat == pNode->beat) {
+				iPixelsPerBeat = pNode->pixelsPerBeat();
+				p.setPen(pal.base().color().value() < 0x7f
+					? pal.light().color() : pal.dark().color()); 
+				p.drawText(x0, y1 + fm.ascent(),
+					QString("%1 %2/%3")
+					.arg(pNode->tempoEx(), 0, 'g', 3)
+					.arg(pNode->beatsPerBar)
+					.arg(1 << pNode->beatDivisor));
+			}
 		} else {
 			y1 = (y2 >> 1);
 		}
-		if (bBeatIsBar || iPixelsPerBeat > 16) {
+		if (bBeatIsBar || iPixelsPerBeat > 8) {
 			p.setPen(pal.mid().color());
 			p.drawLine(x, y1, x, y2);
 			p.setPen(pal.light().color());
 			p.drawLine(x + 1, y1, x + 1, y2);
 		}
-		iFrameFromBeat += iFramesPerBeat;
-		x = pTimeScale->pixelFromFrame(iFrameFromBeat) - dx;
-		iBeat++;
+		pNode = cursor.seekBeat(++iBeat);
+		x = pNode->pixelFromBeat(iBeat) - dx;
 	}
 
 	qtractorSession *pSession = qtractorSession::getInstance();
@@ -389,11 +401,17 @@ void qtractorMidiEditTime::mousePressEvent ( QMouseEvent *pMouseEvent )
 		m_dragState = DragStart;
 		m_posDrag   = pos;
 		// Try to catch mouse clicks over the cursor heads...
-		if (dragHeadStart(m_posDrag))
+		if (dragHeadStart(m_posDrag)) {
 			m_dragState = m_dragCursor;
+		} else if (!bModifier) {
+			// Edit-head positioning...
+			m_pEditor->setEditHead(iFrame);
+			// Logical contents changed, just for visual feedback...
+			m_pEditor->selectionChangeNotify();
+		}
 		break;
 	case Qt::MidButton:
-		// Edit-tail positioning...
+		// Edit-head/tail positioning...
 		m_pEditor->setEditHead(iFrame);
 		m_pEditor->setEditTail(iFrame);
 		// Logical contents changed, just for visual feedback...
@@ -437,6 +455,8 @@ void qtractorMidiEditTime::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 		m_pEditor->editView()->ensureVisible(pos.x(), y, 16, 0);
 		m_pEditor->selectRect(m_rectDrag,
 			pMouseEvent->modifiers() & Qt::ControlModifier, false);
+		// Edit-tail positioning...
+		m_pEditor->setEditTail(iFrame);
 		break;
 	case DragPlayHead:
 		// Play-head positioning...
@@ -495,27 +515,27 @@ void qtractorMidiEditTime::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 		& (Qt::ShiftModifier | Qt::ControlModifier));
 
 	// Direct snap positioning...
+	const QPoint& pos = viewportToContents(pMouseEvent->pos());
 	qtractorTimeScale *pTimeScale = m_pEditor->timeScale();
 	unsigned long iFrame = pTimeScale->frameSnap(m_pEditor->offset()
-		+ pTimeScale->frameFromPixel(m_posDrag.x() > 0 ? m_posDrag.x() : 0));
+		+ pTimeScale->frameFromPixel(pos.x() > 0 ? pos.x() : 0));
 	switch (m_dragState) {
 	case DragSelect:
 		// Do the final range selection...
 		m_pEditor->selectRect(m_rectDrag,
 			pMouseEvent->modifiers() & Qt::ControlModifier, true);
+		// Edit-tail positioning...
+		m_pEditor->setEditTail(iFrame);
+		// Not quite a selection change,
+		// but for visual feedback...
+		m_pEditor->selectionChangeNotify();
 		break;
 	case DragPlayHead:
 		// Play-head positioning commit...
+		m_pEditor->setPlayHead(iFrame);
 		pSession->setPlayHead(m_pEditor->playHead());
-		// Not quite a selection change,
-		// but for visual feedback...
-		m_pEditor->selectionChangeNotify();
-		break;
+		// Fall thru...
 	case DragEditHead:
-		// Not quite a selection change,
-		// but for visual feedback...
-		m_pEditor->selectionChangeNotify();
-		break;
 	case DragEditTail:
 		// Not quite a selection change,
 		// but for visual feedback...
@@ -528,8 +548,6 @@ void qtractorMidiEditTime::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 			m_pEditor->commands()->exec(
 				new qtractorSessionLoopCommand(pSession,
 					pSession->editHead(), pSession->loopEnd()));
-		/*	m_pEditor->updateContents();
-			m_pEditor->contentsChangeNotify(); */
 		}
 		break;
 	case DragLoopEnd:
@@ -539,28 +557,24 @@ void qtractorMidiEditTime::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 			m_pEditor->commands()->exec(
 				new qtractorSessionLoopCommand(pSession,
 					pSession->loopStart(), pSession->editTail()));
-		/*	m_pEditor->updateContents();
-			m_pEditor->contentsChangeNotify(); */
 		}
 		break;
 	case DragPunchIn:
 		// New punch-in boundary...
 		if (pSession->editHead() < pSession->punchOut()) {
 			// Yep, new punch-in point...
-			pSession->setPunch(pSession->editHead(), pSession->punchOut());
-			// For visual feedback...
-			m_pEditor->updateContents();
-			m_pEditor->contentsChangeNotify();
+			m_pEditor->commands()->exec(
+				new qtractorSessionPunchCommand(pSession,
+					pSession->editHead(), pSession->punchOut()));
 		}
 		break;
 	case DragPunchOut:
 		// New punch-out boundary...
 		if (pSession->punchIn() < pSession->editTail()) {
 			// Yep, new punch-out point...
-			pSession->setPunch(pSession->punchIn(), pSession->editTail());
-			// For visual feedback...
-			m_pEditor->updateContents();
-			m_pEditor->contentsChangeNotify();
+			m_pEditor->commands()->exec(
+				new qtractorSessionPunchCommand(pSession,
+					pSession->punchIn(), pSession->editTail()));
 		}
 		break;
 	case DragStart:
@@ -570,13 +584,10 @@ void qtractorMidiEditTime::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 			m_pEditor->setPlayHead(iFrame);
 			// Immediately commited...
 			pSession->setPlayHead(iFrame);
-		} else {
-			// Deferred left-button edit-head positioning...
-			m_pEditor->setEditHead(iFrame);
+			// Not quite a selection, rather just
+			// for immediate visual feedback...
+			m_pEditor->selectionChangeNotify();
 		}
-		// Not quite a selection, rather just
-		// for immediate visual feedback...
-		m_pEditor->selectionChangeNotify();
 		// Fall thru...
 	case DragNone:
 	default:
@@ -588,6 +599,26 @@ void qtractorMidiEditTime::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 }
 
 
+// Tempo-map dialog accessor.
+void qtractorMidiEditTime::mouseDoubleClickEvent ( QMouseEvent *pMouseEvent )
+{
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
+		return;
+
+	// Direct snap positioning...
+	const QPoint& pos = viewportToContents(pMouseEvent->pos());
+	qtractorTimeScale *pTimeScale = m_pEditor->timeScale();
+	unsigned long iFrame = pTimeScale->frameSnap(m_pEditor->offset()
+		+ pTimeScale->frameFromPixel(pos.x() > 0 ? pos.x() : 0));
+
+	// Show tempo map dialog.
+	qtractorTimeScaleForm form(pMainForm);
+	form.setFrame(iFrame);
+	form.exec();
+}
+
+
 // Keyboard event handler.
 void qtractorMidiEditTime::keyPressEvent ( QKeyEvent *pKeyEvent )
 {
@@ -629,9 +660,9 @@ void qtractorMidiEditTime::resetDragState (void)
 
 
 // Context menu event handler (dummy).
-void qtractorMidiEditTime::contextMenuEvent ( QContextMenuEvent * )
+void qtractorMidiEditTime::contextMenuEvent (
+	QContextMenuEvent */*pContextMenuEvent*/ )
 {
-	// Do exactly nothing; ignore event.
 }
 
 
