@@ -19,6 +19,7 @@
 
 *****************************************************************************/
 
+#include "qtractorAbout.h"
 #include "qtractorMidiEditor.h"
 
 #include "qtractorMidiEditor.h"
@@ -1445,6 +1446,18 @@ void qtractorMidiEditor::reset (void)
 }
 
 
+// Clear all contents.
+void qtractorMidiEditor::clear (void)
+{
+	m_pCommands->clear();
+
+	if (m_pMidiClip)
+		m_pMidiClip->sequence()->clear();
+
+	reset();
+}
+
+
 // Intra-clip tick/time positioning reset.
 qtractorMidiEvent *qtractorMidiEditor::seekEvent ( unsigned long iTime )
 {
@@ -1954,8 +1967,8 @@ void qtractorMidiEditor::updateDragSelect ( qtractorScrollView *pScrollView,
 		return;
 
 	// Rubber-banding only applicable whenever
-	// the selection rectangle is not empty...
-	bool bRectSelect = !rectSelect.isEmpty();
+	// the selection rectangle is not that empty...
+	bool bRectSelect = (rectSelect.width() > 1 || rectSelect.height() > 1);
 	if (bRectSelect) {
 		// Create rubber-band, if not already...
 		if (m_pRubberBand == NULL) {
@@ -1970,7 +1983,6 @@ void qtractorMidiEditor::updateDragSelect ( qtractorScrollView *pScrollView,
 	}
 
 	// Do the drag-select update properly...
-
 	bool bEditView
 		= (static_cast<qtractorScrollView *> (m_pEditView) == pScrollView);
 
