@@ -41,6 +41,7 @@
 #include "qtractorClipForm.h"
 
 #include "qtractorTimeScale.h"
+#include "qtractorCommand.h"
 
 #include <QFileDialog>
 #include <QMessageBox>
@@ -469,21 +470,23 @@ qtractorMidiEditorForm::~qtractorMidiEditorForm (void)
 // qtractorMidiEditorForm -- Window close event handlers.
 
 // Pre-close event handlers.
-bool qtractorMidiEditorForm::queryClose (void)
+bool qtractorMidiEditorForm::queryClose ( bool bForce )
 {
 	bool bQueryClose = true;
 
 	// Are we dirty enough to prompt it?
 	if (m_iDirtyCount > 0) {
+		QMessageBox::StandardButtons buttons
+			= QMessageBox::Save
+			| QMessageBox::Discard;
+		if (!bForce)
+			buttons |= QMessageBox::Cancel;
 		switch (QMessageBox::warning(this,
 			tr("Warning") + " - " QTRACTOR_TITLE,
 			tr("The current MIDI clip has been changed:\n\n"
 			"\"%1\"\n\n"
 			"Do you want to save the changes?")
-			.arg(filename()),
-			QMessageBox::Save |
-			QMessageBox::Discard |
-			QMessageBox::Cancel)) {
+			.arg(filename()), buttons)) {
 		case QMessageBox::Save:
 			bQueryClose = saveClipFile(false);
 			// Fall thru....
@@ -528,17 +531,25 @@ bool qtractorMidiEditorForm::queryClose (void)
 }
 
 
-// Unconditional-close.
-void qtractorMidiEditorForm::forceClose (void)
+// (Un)conditional-close.
+bool qtractorMidiEditorForm::testClose ( bool bForce )
 {
 	// Give it a chance to save....
-	queryClose();
+	bool bQueryClose = queryClose(bForce);
 
-	// Make it clean...
-	m_iDirtyCount = 0;
+	if (bQueryClose || bForce) {
+		// Make it clean...
+		m_iDirtyCount = 0;
+		// Close it down!
+		close();
+	}
 
-	// Close it down!
-	close();
+	return bQueryClose;
+}
+
+bool qtractorMidiEditorForm::forceClose (void)
+{
+	return testClose(true);
 }
 
 
@@ -564,8 +575,11 @@ void qtractorMidiEditorForm::closeEvent ( QCloseEvent *pCloseEvent )
 		// Should always (re)open the clip...
 		qtractorMidiClip *pMidiClip = m_pMidiEditor->midiClip();
 		if (pMidiClip && pMidiClip->isDirty()) {
-			 // Revert to original clip length...
+			// Revert to original clip length...
+			pMidiClip->setDirty(false);
 			pMidiClip->setClipLength(m_pMidiEditor->clipLength());
+			// Restart from scratch...
+			m_pMidiEditor->clear();
 			pMidiClip->openMidiFile(
 				pMidiClip->filename(),
 				pMidiClip->trackChannel());
@@ -722,6 +736,9 @@ void qtractorMidiEditorForm::setup ( qtractorMidiClip *pMidiClip )
 
 	// Done.
 	stabilizeForm();
+
+	// Almost.
+	m_pMidiEditor->editView()->setFocus();
 }
 
 
@@ -1199,7 +1216,7 @@ void qtractorMidiEditorForm::viewSnap (void)
 		snapPerBeatChanged(pAction->data().toInt());
 		// Update the other toolbar control...
 		qtractorTimeScale *pTimeScale = m_pMidiEditor->timeScale();
-		if (pTimeScale) 
+		if (pTimeScale)
 			m_pSnapPerBeatComboBox->setCurrentIndex(
 				qtractorTimeScale::indexFromSnap(pTimeScale->snapPerBeat()));
 	}
