@@ -76,13 +76,18 @@ public:
 	void updateInstrumentNames();
 
 	// Pre-close event handler.
-	bool queryClose();
+	bool queryClose(bool bForce = false);
 
-	// Unconditional-close.
-	void forceClose();
+	// (Un)conditional-close.
+	bool testClose(bool bForce = false);
+	bool forceClose();
 
 public slots:
 
+	void stabilizeForm();
+
+protected slots:
+
 	void fileSave();
 	void fileSaveAs();
 	void fileTrackInputs();
@@ -146,10 +151,6 @@ public slots:
 	void selectionChanged(qtractorMidiEditor *);
 	void contentsChanged(qtractorMidiEditor *);
 
-	void stabilizeForm();
-
-protected slots:
-
 	// Event type selection slots.
 	void viewTypeChanged(int);
 	void eventTypeChanged(int);
