@@ -32,10 +32,13 @@
 #include "qtractorMidiClip.h"
 #include "qtractorMidiBuffer.h"
 #include "qtractorMidiControl.h"
+#include "qtractorMidiTimer.h"
+#include "qtractorMidiSysex.h"
 
 #include "qtractorPlugin.h"
 
 #include <QApplication>
+#include <QFileInfo>
 
 #include <QThread>
 #include <QMutex>
@@ -539,6 +542,7 @@ qtractorMidiEngine::qtractorMidiEngine ( qtractorSession *pSession )
 	m_pAlsaSeq       = NULL;
 	m_iAlsaClient    = -1;
 	m_iAlsaQueue     = -1;
+	m_iAlsaTimer     = 0;
 
 	m_pAlsaSubsSeq   = NULL;
 	m_iAlsaSubsPort  = -1;
@@ -742,18 +746,18 @@ void qtractorMidiEngine::resetAllControllers ( bool bForceImmediate )
 		qtractorMidiBus *pMidiBus
 			= static_cast<qtractorMidiBus *> (pBus);
 		if (pMidiBus) {
-			qtractorMidiMonitor *pInputMonitor  = pMidiBus->midiMonitor_in();
 			qtractorMidiMonitor *pOutputMonitor = pMidiBus->midiMonitor_out();
-			if (pInputMonitor) {
-				if (pOutputMonitor == NULL) {
+			if (pOutputMonitor) {
+				pMidiBus->sendSysexList(); // SysEx setup!
+				pMidiBus->setMasterVolume(pOutputMonitor->gain());
+				pMidiBus->setMasterPanning(pOutputMonitor->panning());
+			} else {
+				qtractorMidiMonitor *pInputMonitor = pMidiBus->midiMonitor_in();
+				if (pInputMonitor) {
 					pMidiBus->setMasterVolume(pInputMonitor->gain());
 					pMidiBus->setMasterPanning(pInputMonitor->panning());
 				}
 			}
-			if (pOutputMonitor) {
-				pMidiBus->setMasterVolume(pOutputMonitor->gain());
-				pMidiBus->setMasterPanning(pOutputMonitor->panning());
-			}
 		}
 	}
 
@@ -1179,7 +1183,8 @@ void qtractorMidiEngine::drift (void)
 		iAudioFrame += readAhead();
 		long iDeltaMax = long(pNode->tickFromFrame(iAudioFrame)) - iAudioTime;
 		long iDeltaTime = (iAudioTime - iMidiTime) - m_iTimeDrift;
-		if (iDeltaTime && iDeltaTime > -iDeltaMax && iDeltaTime < +iDeltaMax) {
+		if (iAudioTime && iMidiTime &&
+			iDeltaTime && iDeltaTime > -iDeltaMax && iDeltaTime < +iDeltaMax) {
 		//	m_iTimeStart += iDeltaTime;
 			m_iTimeDrift += iDeltaTime;
 		//	m_iTimeDrift >>= 1; // Damp fast-average drift.
@@ -1225,6 +1230,22 @@ bool qtractorMidiEngine::init ( const QString& sClientName )
 	m_iAlsaClient = snd_seq_client_id(m_pAlsaSeq);
 	m_iAlsaQueue  = snd_seq_alloc_queue(m_pAlsaSeq);
 
+	// Set sequencer queue timer.
+	if (qtractorMidiTimer().indexOf(m_iAlsaTimer) > 0) {
+		qtractorMidiTimer::Key key(m_iAlsaTimer);	
+		snd_timer_id_t *pAlsaTimerId;
+		snd_timer_id_alloca(&pAlsaTimerId);
+		snd_timer_id_set_class(pAlsaTimerId, key.alsaTimerClass());
+		snd_timer_id_set_card(pAlsaTimerId, key.alsaTimerCard());
+		snd_timer_id_set_device(pAlsaTimerId, key.alsaTimerDevice());
+		snd_timer_id_set_subdevice(pAlsaTimerId, key.alsaTimerSubDev());
+		snd_seq_queue_timer_t *pAlsaTimer;
+		snd_seq_queue_timer_alloca(&pAlsaTimer);
+		snd_seq_queue_timer_set_type(pAlsaTimer, SND_SEQ_TIMER_ALSA);
+		snd_seq_queue_timer_set_id(pAlsaTimer, pAlsaTimerId);
+		snd_seq_set_queue_timer(m_pAlsaSeq, m_iAlsaQueue, pAlsaTimer);
+	}
+
 	// Setup subscriptions stuff...
 	if (snd_seq_open(&m_pAlsaSubsSeq, "hw", SND_SEQ_OPEN_DUPLEX, 0) >= 0) {
 		m_iAlsaSubsPort = snd_seq_create_simple_port(
@@ -1650,8 +1671,10 @@ bool qtractorMidiEngine::openControlBus (void)
 		return false;
 
 	// This is it, when dedicated...
-	if (m_bControlBus)
+	if (m_bControlBus) {
+		addBusEx(m_pOControlBus);
 		m_pOControlBus->open();
+	}
 
 	return true;
 }
@@ -1660,20 +1683,22 @@ bool qtractorMidiEngine::openControlBus (void)
 // Close MIDI control stuff.
 void qtractorMidiEngine::closeControlBus (void)
 {
-	if (m_pOControlBus && m_bControlBus)
+	if (m_pOControlBus && m_bControlBus) {
+		removeBusEx(m_pOControlBus);
 		m_pOControlBus->close();
+	}
 }
 
 
 // Destroy MIDI control stuff.
 void qtractorMidiEngine::deleteControlBus (void)
 {
+	closeMetroBus();
+
 	// When owned, both input and output
 	// bus are the one and the same...
-	if (m_pOControlBus && m_bControlBus) {
-		m_pOControlBus->close();
+	if (m_pOControlBus && m_bControlBus)
 		delete m_pOControlBus;
-	}
 
 	// Reset both control buses...
 	m_pIControlBus = NULL;
@@ -1866,8 +1891,10 @@ bool qtractorMidiEngine::openMetroBus (void)
 		return false;
 
 	// This is it, when dedicated...
-	if (m_bMetroBus)
+	if (m_bMetroBus) {
+		addBusEx(m_pMetroBus);
 		m_pMetroBus->open();
+	}
 
 	return true;
 }
@@ -1876,8 +1903,10 @@ bool qtractorMidiEngine::openMetroBus (void)
 // Close MIDI metronome stuff.
 void qtractorMidiEngine::closeMetroBus (void)
 {
-	if (m_pMetroBus && m_bMetroBus)
+	if (m_pMetroBus && m_bMetroBus) {
+		removeBusEx(m_pMetroBus);
 		m_pMetroBus->close();
+	}
 }
 
 
@@ -2308,6 +2337,16 @@ bool qtractorMidiEngine::fileExport ( const QString& sExportPath,
 	bool bResult = file.open(sExportPath, qtractorMidiFile::Write);
 	if (bResult) {
 		if (file.writeHeader(iFormat, iTracks, iTicksPerBeat)) {
+			// Export SysEx setup...	
+			qtractorMidiSysexList *pSysexList = pExportBus->sysexList();
+			if (pSysexList && pSysexList->count() > 0) {
+				if (ppSeqs[0] == NULL) {
+					ppSeqs[0] = new qtractorMidiSequence(
+						QFileInfo(sExportPath).baseName(), 0, iTicksPerBeat);
+				}
+				pExportBus->exportSysexList(ppSeqs[0]);
+			}
+			// Export tempo map as well...	
 			if (file.tempoMap()) {
 				file.tempoMap()->fromTimeScale(
 					pSession->timeScale(), iTimeStart);
@@ -2318,8 +2357,10 @@ bool qtractorMidiEngine::fileExport ( const QString& sExportPath,
 	}
 
 	// Free locally allocated track/sequence array.
-	for (iSeq = 0; iSeq < iSeqs; ++iSeq)
-		delete ppSeqs[iSeq];
+	for (iSeq = 0; iSeq < iSeqs; ++iSeq) {
+		if (ppSeqs[iSeq])
+			delete ppSeqs[iSeq];
+	}
 	delete [] ppSeqs;
 
 	// Done successfully.
@@ -2331,31 +2372,9 @@ bool qtractorMidiEngine::fileExport ( const QString& sExportPath,
 // return the total number of effective (re)connection attempts...
 int qtractorMidiEngine::updateConnects (void)
 {
-	// It must be activated, sure...
-	if (!isActivated())
-		return 0;
-
-	// Do it first on all standard owned dependable buses...
+	// Do it as usual, on all standard owned dependable buses...
 	int iUpdate = qtractorEngine::updateConnects();
 
-	// Control bus inputs...
-	if (m_bControlBus && m_pIControlBus) {
-		iUpdate += m_pIControlBus->updateConnects(
-			qtractorBus::Input, m_pIControlBus->inputs(), true);
-	}
-
-	// Control bus outputs...
-	if (m_bControlBus && m_pOControlBus) {
-		iUpdate += m_pOControlBus->updateConnects(
-			qtractorBus::Output, m_pOControlBus->outputs(), true);
-	}
-
-	// Metronome bus outputs...
-	if (m_bMetroBus && m_pMetroBus) {
-		iUpdate += m_pMetroBus->updateConnects(
-			qtractorBus::Output, m_pMetroBus->outputs(), true);
-	}
-
 	// Reset all pending controllers, if any...
 	if (m_iResetAllControllers > 0)
 		resetAllControllers(true); // Force immediate!
@@ -2391,7 +2410,7 @@ qtractorMidiBus::qtractorMidiBus ( qtractorMidiEngine *pMidiEngine,
 
 	if (busMode & qtractorBus::Input) {
 		m_pIMidiMonitor = new qtractorMidiMonitor();
-		m_pIPluginList  = createPluginList();
+		m_pIPluginList  = createPluginList(qtractorPluginList::MidiInBus);
 	} else {
 		m_pIMidiMonitor = NULL;
 		m_pIPluginList  = NULL;
@@ -2399,10 +2418,12 @@ qtractorMidiBus::qtractorMidiBus ( qtractorMidiEngine *pMidiEngine,
 
 	if (busMode & qtractorBus::Output) {
 		m_pOMidiMonitor = new qtractorMidiMonitor();
-		m_pOPluginList  = createPluginList();
+		m_pOPluginList  = createPluginList(qtractorPluginList::MidiOutBus);
+		m_pSysexList    = new qtractorMidiSysexList();
 	} else {
 		m_pOMidiMonitor = NULL;
 		m_pOPluginList  = NULL;
+		m_pSysexList    = NULL;
 	}
 }
 
@@ -2420,6 +2441,9 @@ qtractorMidiBus::~qtractorMidiBus (void)
 		delete m_pIPluginList;
 	if (m_pOPluginList)
 		delete m_pOPluginList;
+
+	if (m_pSysexList)
+		delete m_pSysexList;
 }
 
 
@@ -2472,14 +2496,10 @@ bool qtractorMidiBus::open (void)
 		return false;
 
 	// Plugin lists need some buffer (re)allocation too...
-	if (m_pIPluginList) {
-		updatePluginList(m_pIPluginList);
-		m_pIPluginList->setName(QObject::tr("%1 In").arg(busName()));
-	}
-	if (m_pOPluginList) {
-		updatePluginList(m_pOPluginList);
-		m_pOPluginList->setName(QObject::tr("%1 Out").arg(busName()));
-	}
+	if (m_pIPluginList)
+		updatePluginList(m_pIPluginList, qtractorPluginList::MidiInBus);
+	if (m_pOPluginList)
+		updatePluginList(m_pOPluginList, qtractorPluginList::MidiOutBus);
 
 	// Done.
 	return true;
@@ -2512,7 +2532,7 @@ void qtractorMidiBus::updateBusMode (void)
 		if (m_pIMidiMonitor == NULL)
 			m_pIMidiMonitor = new qtractorMidiMonitor();
 		if (m_pIPluginList == NULL)
-			m_pIPluginList = createPluginList();
+			m_pIPluginList = createPluginList(qtractorPluginList::MidiInBus);
 	} else {
 		if (m_pIMidiMonitor) {
 			delete m_pIMidiMonitor;
@@ -2529,7 +2549,9 @@ void qtractorMidiBus::updateBusMode (void)
 		if (m_pOMidiMonitor == NULL)
 			m_pOMidiMonitor = new qtractorMidiMonitor();
 		if (m_pOPluginList == NULL)
-			m_pOPluginList = createPluginList();
+			m_pOPluginList = createPluginList(qtractorPluginList::MidiOutBus);
+		if (m_pSysexList == NULL)
+			m_pSysexList = new qtractorMidiSysexList();
 	} else {
 		if (m_pOMidiMonitor) {
 			delete m_pOMidiMonitor;
@@ -2539,6 +2561,10 @@ void qtractorMidiBus::updateBusMode (void)
 			delete m_pOPluginList;
 			m_pOPluginList = NULL;
 		}
+		if (m_pSysexList) {
+			delete m_pSysexList;
+			m_pSysexList = NULL;
+		}
 	}
 }
 
@@ -2568,6 +2594,25 @@ void qtractorMidiBus::shutOff ( bool bClose ) const
 }
 
 
+// Default instrument name accessors.
+void qtractorMidiBus::setInstrumentName ( const QString& sInstrumentName )
+{
+	m_sInstrumentName = sInstrumentName;
+}
+
+const QString& qtractorMidiBus::instrumentName (void) const
+{
+	return m_sInstrumentName;
+}
+
+
+// SysEx setup list accessors.
+qtractorMidiSysexList *qtractorMidiBus::sysexList (void) const
+{
+	return m_pSysexList;
+}
+
+
 // Direct MIDI bank/program selection helper.
 void qtractorMidiBus::setPatch ( unsigned short iChannel,
 	const QString& sInstrumentName, int iBankSelMethod,
@@ -2789,7 +2834,7 @@ void qtractorMidiBus::sendNote ( qtractorTrack *pTrack,
 }
 
 
-// Direct SysEx helper.
+// Direct SysEx helpers.
 void qtractorMidiBus::sendSysex ( unsigned char *pSysex, unsigned int iSysex ) const
 {
 	// Yet again, we need our MIDI engine reference...
@@ -2804,9 +2849,9 @@ void qtractorMidiBus::sendSysex ( unsigned char *pSysex, unsigned int iSysex ) c
 		return;
 
 #ifdef CONFIG_DEBUG_0
-	fprintf(stderr, "qtractorMidiBus::sendSysEx(%p, %u)", pSysex, iSysex);
+	fprintf(stderr, "qtractorMidiBus::sendSysex(%p, %u)", pSysex, iSysex);
 	fprintf(stderr, " sysex {");
-	for (unsigned int i = 0; i < iSysex; i++)
+	for (unsigned int i = 0; i < iSysex; ++i)
 		fprintf(stderr, " %02x", pSysex[i]);
 	fprintf(stderr, " }\n");
 #endif
@@ -2831,6 +2876,55 @@ void qtractorMidiBus::sendSysex ( unsigned char *pSysex, unsigned int iSysex ) c
 }
 
 
+void qtractorMidiBus::sendSysexList (void) const
+{
+	// Check that we have some SysEx for setup...
+	if (m_pSysexList == NULL)
+		return;
+	if (m_pSysexList->count() < 1)
+		return;
+
+	// Yet again, we need our MIDI engine reference...
+	qtractorMidiEngine *pMidiEngine
+		= static_cast<qtractorMidiEngine *> (engine());
+	if (pMidiEngine == NULL)
+		return;
+
+	// Don't do anything else if engine
+	// has not been activated...
+	if (pMidiEngine->alsaSeq() == NULL)
+		return;
+
+	QListIterator<qtractorMidiSysex *> iter(*m_pSysexList);
+	while (iter.hasNext()) {
+		qtractorMidiSysex *pSysex = iter.next();
+	#ifdef CONFIG_DEBUG_0
+		unsigned char *pData = pSysex->data();
+		unsigned short iSize = pSysex->size();
+		fprintf(stderr, "qtractorMidiBus::sendSysexList(%p, %u)", pData, iSize);
+		fprintf(stderr, " sysex {");
+		for (unsigned short i = 0; i < iSize; ++i)
+			fprintf(stderr, " %02x", pData[i]);
+		fprintf(stderr, " }\n");
+	#endif
+		// Initialize sequencer event...
+		snd_seq_event_t ev;
+		snd_seq_ev_clear(&ev);
+		// Addressing...
+		snd_seq_ev_set_source(&ev, m_iAlsaPort);
+		snd_seq_ev_set_subs(&ev);
+		// The event will be direct...
+		snd_seq_ev_set_direct(&ev);
+		// Just set SYSEX stuff and send it out..
+		ev.type = SND_SEQ_EVENT_SYSEX;
+		snd_seq_ev_set_sysex(&ev, pSysex->size(), pSysex->data());
+		snd_seq_event_output(pMidiEngine->alsaSeq(), &ev);
+	}
+
+	pMidiEngine->flush();
+}
+
+
 // Virtual I/O bus-monitor accessors.
 qtractorMonitor *qtractorMidiBus::monitor_in (void) const
 {
@@ -2868,7 +2962,7 @@ qtractorPluginList *qtractorMidiBus::pluginList_out (void) const
 
 
 // Create plugin-list properly.
-qtractorPluginList *qtractorMidiBus::createPluginList (void) const
+qtractorPluginList *qtractorMidiBus::createPluginList ( int iFlags ) const
 {
 	qtractorSession *pSession = engine()->session();
 	if (pSession == NULL)
@@ -2888,24 +2982,36 @@ qtractorPluginList *qtractorMidiBus::createPluginList (void) const
 		}
 	}
 
-	// Got it?
-	if (pAudioBus == NULL)
-		return new qtractorPluginList(0, 0, 0, qtractorPluginList::MidiBus);
-
 	// Create plugin-list alright...
-	return new qtractorPluginList(pAudioBus->channels(),
-		pAudioEngine->bufferSize(), pAudioEngine->sampleRate(),
-		qtractorPluginList::MidiBus);
+	qtractorPluginList *pPluginList = NULL;
+	if (pAudioBus) {
+	 	pPluginList = new qtractorPluginList(pAudioBus->channels(),
+			pAudioEngine->bufferSize(), pAudioEngine->sampleRate(), iFlags);
+	} else {
+		pPluginList = new qtractorPluginList(0, 0, 0, iFlags);
+	}
+
+	// Set plugin-list title name...
+	updatePluginListName(pPluginList, iFlags);
+
+	return pPluginList;
+}
+
+
+// Update plugin-list title name...
+void qtractorMidiBus::updatePluginListName (
+	qtractorPluginList *pPluginList, int iFlags ) const
+{
+	pPluginList->setName((iFlags & qtractorPluginList::In ?
+		QObject::tr("%1 In") : QObject::tr("%1 Out")).arg(busName()));
 }
 
 
 // Update plugin-list buffers properly.
-void qtractorMidiBus::updatePluginList ( qtractorPluginList *pPluginList )
+void qtractorMidiBus::updatePluginList (
+	qtractorPluginList *pPluginList, int iFlags )
 {
 	// Sanity checks...
-	if (pPluginList == NULL)
-		return;
-
 	qtractorSession *pSession = engine()->session();
 	if (pSession == NULL)
 		return;
@@ -2913,7 +3019,10 @@ void qtractorMidiBus::updatePluginList ( qtractorPluginList *pPluginList )
 	qtractorAudioEngine *pAudioEngine = pSession->audioEngine();
 	if (pAudioEngine == NULL)
 		return;
-	
+
+	// Set plugin-list title name...
+	updatePluginListName(pPluginList, iFlags);
+
 	// Get audio bus as for the plugin list...
 	qtractorAudioBus *pAudioBus = NULL;
 	if (pPluginList->midiManager())
@@ -2935,8 +3044,7 @@ void qtractorMidiBus::updatePluginList ( qtractorPluginList *pPluginList )
 
 	// Set plugin-list buffer alright...
 	pPluginList->setBuffer(pAudioBus->channels(),
-		pAudioEngine->bufferSize(), pAudioEngine->sampleRate(),
-		qtractorPluginList::MidiBus);
+		pAudioEngine->bufferSize(), pAudioEngine->sampleRate(), iFlags);
 }
 
 
@@ -2986,7 +3094,8 @@ int qtractorMidiBus::updateConnects ( qtractorBus::BusMode busMode,
 	while (snd_seq_query_next_client(
 			pMidiEngine->alsaSeq(), pClientInfo) >= 0) {
 		item.client = snd_seq_client_info_get_client(pClientInfo);
-		item.clientName = snd_seq_client_info_get_name(pClientInfo);
+		item.clientName = QString::fromUtf8(
+			snd_seq_client_info_get_name(pClientInfo));
 		snd_seq_port_info_set_client(pPortInfo, item.client);
 		snd_seq_port_info_set_port(pPortInfo, -1);
 		while (snd_seq_query_next_port(
@@ -2996,7 +3105,8 @@ int qtractorMidiBus::updateConnects ( qtractorBus::BusMode busMode,
 			if (((iPortCapability & iPortFlags) == iPortFlags) &&
 				((iPortCapability & SND_SEQ_PORT_CAP_NO_EXPORT) == 0)) {
 				item.port = snd_seq_port_info_get_port(pPortInfo);
-				item.portName = snd_seq_port_info_get_name(pPortInfo);
+				item.portName = QString::fromUtf8(
+					snd_seq_port_info_get_name(pPortInfo));
 				pItem = connects.findItem(item);
 				if (pItem) {
 					pItem->port = item.port;
@@ -3018,11 +3128,13 @@ int qtractorMidiBus::updateConnects ( qtractorBus::BusMode busMode,
 		snd_seq_get_any_client_info(
 			pMidiEngine->alsaSeq(), seq_addr.client, pClientInfo);
 		item.client = seq_addr.client;
-		item.clientName = snd_seq_client_info_get_name(pClientInfo);
+		item.clientName = QString::fromUtf8(
+			snd_seq_client_info_get_name(pClientInfo));
 		snd_seq_get_any_port_info(
 			pMidiEngine->alsaSeq(), seq_addr.client, seq_addr.port, pPortInfo);
 		item.port = seq_addr.port;
-		item.portName = snd_seq_port_info_get_name(pPortInfo);
+		item.portName = QString::fromUtf8(
+			snd_seq_port_info_get_name(pPortInfo));
 		// Check if already in list/connected...
 		pItem = connects.findItem(item);
 		if (pItem && bConnect) {
@@ -3162,8 +3274,12 @@ bool qtractorMidiBus::loadElement ( qtractorSessionDocument *pDocument,
 			eProp.tagName() == "midi-thru") { // Legacy compat.
 			qtractorMidiBus::setPassthru(
 				pDocument->boolFromText(eProp.text()));
+		} else if (eProp.tagName() == "midi-sysex-list") {
+			qtractorMidiBus::loadSysexList(pDocument, &eProp);
 		} else if (eProp.tagName() == "midi-map") {
 			qtractorMidiBus::loadMidiMap(pDocument, &eProp);
+		} else if (eProp.tagName() == "midi-instrument-name") {
+			qtractorMidiBus::setInstrumentName(eProp.text());
 		} else if (eProp.tagName() == "input-gain") {
 			if (qtractorMidiBus::monitor_in())
 				qtractorMidiBus::monitor_in()->setGain(
@@ -3256,11 +3372,27 @@ bool qtractorMidiBus::saveElement ( qtractorSessionDocument *pDocument,
 		pElement->appendChild(eMidiOutputs);
 	}
 
+	// Save default intrument name, if any...
+	if (!qtractorMidiBus::instrumentName().isEmpty()) {
+		pDocument->saveTextElement("midi-instrument-name",
+			qtractorMidiBus::instrumentName(), pElement);
+	}
+
+	// Create the sysex element...
+	if (m_pSysexList && m_pSysexList->count() > 0) {
+		QDomElement eSysexList
+			= pDocument->document()->createElement("midi-sysex-list");
+		qtractorMidiBus::saveSysexList(pDocument, &eSysexList);
+		pElement->appendChild(eSysexList);
+	}
+
 	// Create the map element...
-	QDomElement eMidiMap
-		= pDocument->document()->createElement("midi-map");
-	qtractorMidiBus::saveMidiMap(pDocument, &eMidiMap);
-	pElement->appendChild(eMidiMap);
+	if (m_patches.count() > 0) {
+		QDomElement eMidiMap
+			= pDocument->document()->createElement("midi-map");
+		qtractorMidiBus::saveMidiMap(pDocument, &eMidiMap);
+		pElement->appendChild(eMidiMap);
+	}
 
 	return true;
 }
@@ -3348,4 +3480,107 @@ bool qtractorMidiBus::saveMidiMap ( qtractorSessionDocument *pDocument,
 }
 
 
+// Document SysEx setup list methods.
+bool qtractorMidiBus::loadSysexList ( qtractorSessionDocument * /*pDocument*/,
+	QDomElement *pElement )
+{
+	// Must have one...
+	if (m_pSysexList == NULL)
+		return false;
+
+	// Crystal clear...
+	m_pSysexList->clear();
+
+	// Load map items...
+	for (QDomNode nChild = pElement->firstChild();
+			!nChild.isNull();
+				nChild = nChild.nextSibling()) {
+
+		// Convert node to element...
+		QDomElement eChild = nChild.toElement();
+		if (eChild.isNull())
+			continue;
+
+		// Load map item...
+		if (eChild.tagName() == "midi-sysex") {
+			qtractorMidiSysex *pSysex
+				= new qtractorMidiSysex(
+					eChild.attribute("name"), eChild.text());
+			if (pSysex->size() > 0)
+				m_pSysexList->append(pSysex);
+			else
+				delete pSysex;
+		}
+	}
+
+	return true;
+}
+
+
+bool qtractorMidiBus::saveSysexList ( qtractorSessionDocument *pDocument,
+	QDomElement *pElement )
+{
+	// Must have one...
+	if (m_pSysexList == NULL)
+		return false;
+
+	// Save map items...
+	QListIterator<qtractorMidiSysex *> iter(*m_pSysexList);
+	while (iter.hasNext()) {
+		qtractorMidiSysex *pSysex = iter.next();
+		QDomElement eSysex = pDocument->document()->createElement("midi-sysex");
+		eSysex.setAttribute("name", pSysex->name());
+		eSysex.appendChild(
+			pDocument->document()->createTextNode(pSysex->text()));
+		pElement->appendChild(eSysex);
+	}
+
+	return true;
+}
+
+
+// Import SysEx setup from event sequence.
+bool qtractorMidiBus::importSysexList ( qtractorMidiSequence *pSeq )  
+{
+	if (m_pSysexList == NULL)
+		return false;
+
+	m_pSysexList->clear();
+
+	int iSysex = 0;
+	qtractorMidiEvent *pEvent = pSeq->events().first();
+	while (pEvent) {
+		if (pEvent->type() == qtractorMidiEvent::SYSEX) {
+			m_pSysexList->append(
+				new qtractorMidiSysex(pSeq->name()
+					+ '-' + QString::number(++iSysex),
+					pEvent->sysex(), pEvent->sysex_len())
+			);
+		}
+		pEvent = pEvent->next();
+	}
+
+	return true;
+}
+
+
+// Export SysEx setup to event sequence.
+bool qtractorMidiBus::exportSysexList ( qtractorMidiSequence *pSeq )
+{
+	if (m_pSysexList == NULL)
+		return false;
+
+	QListIterator<qtractorMidiSysex *> iter(*m_pSysexList);
+	while (iter.hasNext()) {
+		qtractorMidiSysex *pSysex = iter.next();
+		qtractorMidiEvent *pEvent
+			= new qtractorMidiEvent(0, qtractorMidiEvent::SYSEX);
+		pEvent->setSysex(pSysex->data(), pSysex->size());
+		pSeq->addEvent(pEvent);			
+	}
+
+	return true;
+}
+
+
 // end of qtractorMidiEngine.cpp
