@@ -1,7 +1,7 @@
 // qtractorMidiEvent.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -109,6 +109,10 @@ public:
 	void setTime(unsigned long time) { m_time = time; }
 	void setType(EventType type)     { m_type = type; }
 
+	// Special event time offset adjust.
+	void adjustTime(unsigned long iOffset)
+		{ m_time = (m_time > iOffset ? m_time - iOffset : 0); }
+
 	// Underloaded accessors (setters).
 	void setNote(unsigned char note)             { m_data[0] = note; }
 	void setVelocity(unsigned char velocity)     { m_data[1] = velocity; }
