@@ -127,6 +127,7 @@ qtractorOptions::qtractorOptions (void)
 	m_settings.beginGroup("/Midi");
 	iMidiCaptureFormat = m_settings.value("/CaptureFormat", 1).toInt();
 	iMidiCaptureQuantize = m_settings.value("/CaptureQuantize", 0).toInt();
+	iMidiQueueTimer    = m_settings.value("/QueueTimer", 0).toInt();
 	bMidiControlBus    = m_settings.value("/ControlBus", false).toBool();
 	bMidiMetroBus      = m_settings.value("/MetroBus", false).toBool();
 	bMidiMetronome     = m_settings.value("/Metronome", true).toBool();
@@ -159,6 +160,7 @@ qtractorOptions::qtractorOptions (void)
 	sPresetDir     = m_settings.value("/PresetDir").toString();
 	sInstrumentDir = m_settings.value("/InstrumentDir").toString();
 	sMidiControlDir = m_settings.value("/MidiControlDir").toString();
+	sMidiSysexDir  = m_settings.value("/MidiSysexDir").toString();
 	bAutoMonitor   = m_settings.value("/AutoMonitor", true).toBool();
 	iSnapPerBeat   = m_settings.value("/SnapPerBeat", 4).toInt();
 	fTempo   = float(m_settings.value("/Tempo", 120.0).toDouble());
@@ -332,6 +334,7 @@ qtractorOptions::~qtractorOptions (void)
 	m_settings.beginGroup("/Midi");
 	m_settings.setValue("/CaptureFormat", iMidiCaptureFormat);
 	m_settings.setValue("/CaptureQuantize", iMidiCaptureQuantize);
+	m_settings.setValue("/QueueTimer", iMidiQueueTimer);
 	m_settings.setValue("/ControlBus", bMidiControlBus);
 	m_settings.setValue("/MetroBus", bMidiMetroBus);
 	m_settings.setValue("/Metronome", bMidiMetronome);
@@ -364,6 +367,7 @@ qtractorOptions::~qtractorOptions (void)
 	m_settings.setValue("/PresetDir", sPresetDir);
 	m_settings.setValue("/InstrumentDir", sInstrumentDir);
 	m_settings.setValue("/MidiControlDir", sMidiControlDir);
+	m_settings.setValue("/MidiSysexDir", sMidiSysexDir);
 	m_settings.setValue("/AutoMonitor", bAutoMonitor);
 	m_settings.setValue("/SnapPerBeat", iSnapPerBeat);
 	m_settings.setValue("/Tempo", double(fTempo));
