@@ -110,6 +110,7 @@ public:
 	// MIDI options...
 	int  iMidiCaptureFormat;
 	int  iMidiCaptureQuantize;
+	int  iMidiQueueTimer;
 	bool bMidiControlBus;
 	bool bMidiMetroBus;
 	bool bMidiMetronome;
@@ -130,6 +131,7 @@ public:
 	QString sPresetDir;
 	QString sInstrumentDir;
 	QString sMidiControlDir;
+	QString sMidiSysexDir;
 
 	// Session options.
 	bool    bSessionTemplate;
