@@ -25,6 +25,7 @@
 #include "qtractorOptions.h"
 
 #include "qtractorAudioFile.h"
+#include "qtractorMidiTimer.h"
 #include "qtractorMidiEditor.h"
 #include "qtractorTimeScale.h"
 #include "qtractorPlugin.h"
@@ -159,6 +160,9 @@ qtractorOptionsForm::qtractorOptionsForm (
 	QObject::connect(m_ui.MidiCaptureQuantizeComboBox,
 		SIGNAL(activated(int)),
 		SLOT(changed()));
+	QObject::connect(m_ui.MidiQueueTimerComboBox,
+		SIGNAL(activated(int)),
+		SLOT(changed()));
 	QObject::connect(m_ui.MidiControlBusCheckBox,
 		SIGNAL(stateChanged(int)),
 		SLOT(changed()));
@@ -362,9 +366,19 @@ void qtractorOptionsForm::setOptions ( qtractorOptions *pOptions )
 	m_ui.MetroBeatFilenameComboBox->setEditText(m_pOptions->sMetroBeatFilename);
 	m_ui.AudioMetroBusCheckBox->setChecked(m_pOptions->bAudioMetroBus);
 
-	// MIDI options.
+	// MIDI capture/export options.
 	m_ui.MidiCaptureFormatComboBox->setCurrentIndex(m_pOptions->iMidiCaptureFormat);
 	m_ui.MidiCaptureQuantizeComboBox->setCurrentIndex(m_pOptions->iMidiCaptureQuantize);
+
+	// MIDI playback options.
+	qtractorMidiTimer timer;
+	m_ui.MidiQueueTimerComboBox->clear();
+	for (int i = 0; i < timer.count(); ++i)
+		m_ui.MidiQueueTimerComboBox->addItem(timer.name(i), timer.key(i));
+	m_ui.MidiQueueTimerComboBox->setCurrentIndex(
+		timer.indexOf(m_pOptions->iMidiQueueTimer));
+
+	// MIDI control options.
 	m_ui.MidiControlBusCheckBox->setChecked(m_pOptions->bMidiControlBus);
 
 	// MIDI metronome options.
@@ -488,6 +502,8 @@ void qtractorOptionsForm::accept (void)
 		// MIDI options...
 		m_pOptions->iMidiCaptureFormat   = m_ui.MidiCaptureFormatComboBox->currentIndex();
 		m_pOptions->iMidiCaptureQuantize = m_ui.MidiCaptureQuantizeComboBox->currentIndex();
+		m_pOptions->iMidiQueueTimer      = m_ui.MidiQueueTimerComboBox->itemData(
+			m_ui.MidiQueueTimerComboBox->currentIndex()).toInt();
 		m_pOptions->bMidiControlBus      = m_ui.MidiControlBusCheckBox->isChecked();
 		// MIDI metronome options.
 		m_pOptions->bMidiMetronome       = m_ui.MidiMetronomeCheckBox->isChecked();
@@ -529,7 +545,6 @@ void qtractorOptionsForm::accept (void)
 		m_pOptions->sSessionTemplatePath = m_ui.SessionTemplatePathComboBox->currentText();
 		// Custom colors.
 		int iColor;
-		m_pOptions->audioMeterColors.clear();
 		for (iColor = 0; iColor < AudioMeterColors; ++iColor)
 			qtractorAudioMeter::setColor(iColor, m_audioMeterColors[iColor]);
 		for (iColor = 0; iColor < MidiMeterColors; ++iColor)
