@@ -41,11 +41,23 @@
 #include "qtractorVstPlugin.h"
 #endif
 
+#include "qtractorInsertPlugin.h"
+
 #include <QDomDocument>
 #include <QTextStream>
 #include <QFileInfo>
 #include <QDir>
 
+#if QT_VERSION < 0x040500
+namespace Qt {
+const WindowFlags WindowCloseButtonHint = WindowFlags(0x08000000);
+#if QT_VERSION < 0x040200
+const WindowFlags CustomizeWindowHint   = WindowFlags(0x02000000);
+#endif
+}
+#endif
+
+
 typedef void (*qtractorPluginFile_Function)(void);
 
 #if defined(__WIN32__) || defined(_WIN32) || defined(WIN32)
@@ -288,6 +300,19 @@ qtractorPlugin *qtractorPluginFile::createPlugin (
 		pList, sFilename.toUtf8().constData(), iIndex, int(typeHint));
 #endif
 
+	// Attend to insert pseudo-plugin hints...
+	if (sFilename.isEmpty() || typeHint == qtractorPluginType::Insert) {
+		qtractorInsertPluginType *pInsertType
+			= qtractorInsertPluginType::createType(iIndex);
+		if (pInsertType) {
+			if (pInsertType->open())
+				return new qtractorInsertPlugin(pList, pInsertType);
+			delete pInsertType;
+		}
+		// Don't bother with anything else.
+		return NULL;
+	}
+
 	// Try to fill the types list at this moment...
 	qtractorPluginFile *pFile = new qtractorPluginFile(sFilename);
 	if (!pFile->open()) {
@@ -384,6 +409,9 @@ qtractorPluginType::Hint qtractorPluginType::hintFromText (
 		return Vst;
 	else
 #endif
+	if (sText == "Insert")
+		return Insert;
+	else
 	return Any;
 }
 
@@ -405,6 +433,9 @@ QString qtractorPluginType::textFromHint (
 		return "VST";
 	else
 #endif
+	if (typeHint == Insert)
+		return "Insert";
+	else
 	return QObject::tr("(Any)");
 }
 
@@ -623,15 +654,11 @@ qtractorPluginForm *qtractorPlugin::form (void)
 		// What style do we create tool childs?
 		QWidget *pParent = NULL;
 		Qt::WindowFlags wflags = Qt::Window
-		#if QT_VERSION >= 0x040200
 			| Qt::CustomizeWindowHint
-		#if QT_VERSION >= 0x040500
-			| Qt::WindowCloseButtonHint
-		#endif
-		#endif
 			| Qt::WindowTitleHint
 			| Qt::WindowSystemMenuHint
-			| Qt::WindowMinMaxButtonsHint;
+			| Qt::WindowMinMaxButtonsHint
+			| Qt::WindowCloseButtonHint;
 		qtractorOptions *pOptions = qtractorOptions::getInstance();
 		if (pOptions && pOptions->bKeepToolsOnTop) {
 		//	pParent = qtractorMainForm::getInstance();
@@ -1160,8 +1187,11 @@ qtractorPlugin *qtractorPluginList::copyPlugin ( qtractorPlugin *pPlugin )
 	if (m_pMidiManager && m_pMidiManager->currentProg() >= 0)
 		iProg = m_pMidiManager->currentProg();
 
+	QString sFilename; // Filname is empty for insert pseudo-plugins.
+	if (pType->file())
+		sFilename = (pType->file())->filename();
 	qtractorPlugin *pNewPlugin = qtractorPluginFile::createPlugin(this,
-		(pType->file())->filename(), pType->index(), pType->typeHint());
+		sFilename, pType->index(), pType->typeHint());
 	if (pNewPlugin) {
 		pNewPlugin->setPreset(pPlugin->preset());
 		pNewPlugin->setConfigs(pPlugin->configs());
@@ -1282,6 +1312,7 @@ bool qtractorPluginList::loadElement ( qtractorSessionDocument *pDocument,
 		if (ePlugin.tagName() == "plugin") {
 			QString sFilename;
 			unsigned long iIndex = 0;
+			QString sLabel;
 			QString sPreset;
 			QStringList vlist;
 			bool bActivated = false;
@@ -1303,6 +1334,9 @@ bool qtractorPluginList::loadElement ( qtractorSessionDocument *pDocument,
 				if (eParam.tagName() == "index")
 					iIndex = eParam.text().toULong();
 				else
+				if (eParam.tagName() == "label")
+					sLabel = eParam.text();
+				else
 				if (eParam.tagName() == "preset")
 					sPreset = eParam.text();
 				else
@@ -1322,11 +1356,17 @@ bool qtractorPluginList::loadElement ( qtractorSessionDocument *pDocument,
 					qtractorPlugin::loadValues(&eParam, values);
 				}
 			}
-			if (sFilename.isEmpty())
-				continue;
 			qtractorPlugin *pPlugin
 				= qtractorPluginFile::createPlugin(this,
 					sFilename, iIndex, typeHint);
+			if (!sFilename.isEmpty() && !sLabel.isEmpty() &&
+				((pPlugin == NULL) || ((pPlugin->type())->label() != sLabel))) {
+				iIndex = 0;
+				do {
+					pPlugin = qtractorPluginFile::createPlugin(this,
+						sFilename, iIndex++, typeHint);
+				} while (pPlugin && (pPlugin->type())->label() != sLabel);
+			}
 			if (pPlugin) {
 				pPlugin->setPreset(sPreset);
 				pPlugin->setConfigs(configs);
@@ -1390,12 +1430,17 @@ bool qtractorPluginList::saveElement ( qtractorSessionDocument *pDocument,
 
 		// Create the new plugin element...
 		QDomElement ePlugin = pDocument->document()->createElement("plugin");
-		ePlugin.setAttribute("type", qtractorPluginType::textFromHint(
-			(pPlugin->type())->typeHint()));
-		pDocument->saveTextElement("filename",
-			((pPlugin->type())->file())->filename(), &ePlugin);
+		qtractorPluginType *pType = pPlugin->type();
+		ePlugin.setAttribute("type",
+			qtractorPluginType::textFromHint(pType->typeHint()));
+		if (pType->file()) { // Pseudo-plugins don't have a file...
+			pDocument->saveTextElement("filename",
+				(pType->file())->filename(), &ePlugin);
+		}
 		pDocument->saveTextElement("index",
-			QString::number((pPlugin->type())->index()), &ePlugin);
+			QString::number(pType->index()), &ePlugin);
+		pDocument->saveTextElement("label",
+			pType->label(), &ePlugin);
 		pDocument->saveTextElement("preset",
 			pPlugin->preset(), &ePlugin);
 	//	pDocument->saveTextElement("values",
