@@ -58,7 +58,7 @@ class qtractorPluginType
 public:
 
 	// Have hints for plugin paths.
-	enum Hint { Any = 0, Ladspa, Dssi, Vst };
+	enum Hint { Any = 0, Ladspa, Dssi, Vst, Insert };
 
 	// Constructor.
 	qtractorPluginType(qtractorPluginFile *pFile, unsigned long iIndex,
@@ -365,9 +365,6 @@ public:
 	virtual void configure(
 		const QString& /*sKey*/, const QString& /*sValue*/) {}
 
-	// Tall whether configuration snapshot is needed.
-	virtual bool isConfigure() { return false; }
-
 	// Plugin configuration/state snapshot.
 	virtual void freezeConfigs() {}
 	virtual void releaseConfigs() {}
@@ -508,11 +505,16 @@ public:
 		// Basic flags.
 		Audio = 0, Midi = 1,
 		Track = 0, Bus  = 2,
+		Out   = 0, In   = 4,
 		// Composite helper flags.
-		AudioTrack = Audio | Track,
-		AudioBus   = Audio | Bus,
-		MidiTrack  = Midi  | Track,
-		MidiBus    = Midi  | Bus
+		AudioTrack  = Audio | Track,
+		AudioBus    = Audio | Bus,
+		AudioInBus  = Audio | Bus | In,
+		AudioOutBus = Audio | Bus | Out,
+		MidiTrack   = Midi  | Track,
+		MidiBus     = Midi  | Bus, 
+		MidiInBus   = Midi  | Bus | In,
+		MidiOutBus  = Midi  | Bus | Out
 	};
 
 	// Constructor.
