@@ -1,7 +1,7 @@
 // qtractorPluginCommand.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -87,6 +87,23 @@ public:
 };
 
 
+//----------------------------------------------------------------------
+// class qtractorAddInsertPluginCommand - declaration.
+//
+
+class qtractorAddInsertPluginCommand : public qtractorPluginCommand
+{
+public:
+
+	// Constructor.
+	qtractorAddInsertPluginCommand(qtractorPlugin *pPlugin = NULL);
+
+	// Plugin insertion command methods.
+	bool redo();
+	bool undo();
+};
+
+
 //----------------------------------------------------------------------
 // class qtractorRemovePluginCommand - declaration.
 //
