@@ -30,6 +30,7 @@
 
 #include "qtractorOptions.h"
 #include "qtractorSession.h"
+#include "qtractorEngine.h"
 
 #include <QMessageBox>
 #include <QGridLayout>
@@ -71,6 +72,9 @@ qtractorPluginForm::qtractorPluginForm (
     m_ui.PresetComboBox->setValidator(
 		new QRegExpValidator(QRegExp("[\\w-]+"), m_ui.PresetComboBox));
 	m_ui.PresetComboBox->setInsertPolicy(QComboBox::NoInsert);
+#if QT_VERSION >= 0x040200
+	m_ui.PresetComboBox->setCompleter(NULL);
+#endif
 
 	// Have some effective feedback when toggling on/off...
 	QIcon iconParams;
@@ -112,6 +116,12 @@ qtractorPluginForm::qtractorPluginForm (
 	QObject::connect(m_ui.EditToolButton,
 		SIGNAL(toggled(bool)),
 		SLOT(editSlot(bool)));
+	QObject::connect(m_ui.SendsToolButton,
+		SIGNAL(clicked()),
+		SLOT(sendsSlot()));
+	QObject::connect(m_ui.ReturnsToolButton,
+		SIGNAL(clicked()),
+		SLOT(returnsSlot()));
 	QObject::connect(m_ui.ActivateToolButton,
 		SIGNAL(toggled(bool)),
 		SLOT(activateSlot(bool)));
@@ -190,6 +200,12 @@ void qtractorPluginForm::setPlugin ( qtractorPlugin *pPlugin )
 	//if (bEditor)
 	//	toggleEditor(m_pPlugin->isEditorVisible());
 
+	// Show insert tool options...
+	bool bInsertPlugin
+		= ((m_pPlugin->type())->typeHint() == qtractorPluginType::Insert);
+	m_ui.SendsToolButton->setVisible(bInsertPlugin);
+	m_ui.ReturnsToolButton->setVisible(bInsertPlugin);
+
 	// Set plugin name as title...
 	updateCaption();
 
@@ -565,11 +581,13 @@ void qtractorPluginForm::deletePresetSlot (void)
 	// Go ahead...
 	QSettings& settings = pOptions->settings();
 	settings.beginGroup(m_pPlugin->presetGroup());
+#ifdef QTRACTOR_REMOVE_PRESET_FILES
 	if ((m_pPlugin->type())->isConfigure()) {
 		const QString& sFilename = settings.value(sPreset).toString();
 		if (QFileInfo(sFilename).exists())
 			QFile(sFilename).remove();
 	}
+#endif
 	settings.remove(sPreset);
 	settings.endGroup();
 	refresh();
@@ -624,6 +642,20 @@ void qtractorPluginForm::editSlot ( bool bOn )
 }
 
 
+// Outputs (Sends) slot.
+void qtractorPluginForm::sendsSlot (void)
+{
+	qtractorPluginListView::insertPluginBus(m_pPlugin, qtractorBus::Output);
+}
+
+
+// Inputs (Returns) slot.
+void qtractorPluginForm::returnsSlot (void)
+{
+	qtractorPluginListView::insertPluginBus(m_pPlugin, qtractorBus::Input);
+}
+
+
 // Activation slot.
 void qtractorPluginForm::activateSlot ( bool bOn )
 {
