@@ -87,6 +87,8 @@ protected slots:
 	void deletePresetSlot();
 	void paramsSlot(bool bOn);
 	void editSlot(bool bOn);
+	void sendsSlot();
+	void returnsSlot();
 	void activateSlot(bool bOn);
 	void valueChangeSlot(qtractorPluginParam *pParam, float fValue);
 
@@ -94,6 +96,9 @@ protected:
 
 	void stabilize();
 
+	// Show insert pseudo-plugin audio bus connections.
+	void insertPluginBus(int iBusMode);
+
 private:
 
 	// The Qt-designer UI struct...
