@@ -1,7 +1,7 @@
 // qtractorSpinBox.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -32,18 +32,18 @@ class QShowEvent;
 
 
 //----------------------------------------------------------------------------
-// qtractorSpinBox -- A time-scale formatted spin-box widget.
+// qtractorTimeSpinBox -- A time-scale formatted spin-box widget.
 
-class qtractorSpinBox : public QAbstractSpinBox
+class qtractorTimeSpinBox : public QAbstractSpinBox
 {
     Q_OBJECT
 
 public:
 
 	// Constructor.
-	qtractorSpinBox(QWidget *pParent = 0);
+	qtractorTimeSpinBox(QWidget *pParent = 0);
 	// Destructor.
-	~qtractorSpinBox();
+	~qtractorTimeSpinBox();
 
 	// Time-scale accessors.
 	void setTimeScale(qtractorTimeScale *pTimeScale);
@@ -66,8 +66,9 @@ public:
 	unsigned long maximum() const;
 
 	// Differential value mode (BBT format only) accessor.
-	void setDeltaValue(bool bDeltaValue);
+	void setDeltaValue(bool bDeltaValue, unsigned long iDeltaValue = 0);
 	bool isDeltaValue() const;
+	unsigned long deltaValue() const;
 
 signals:
 
@@ -103,10 +104,77 @@ private:
 	unsigned long      m_iDefaultValue;
 	unsigned long      m_iMinimumValue;
 	unsigned long      m_iMaximumValue;
+	unsigned long      m_iDeltaValue;
 	bool               m_bDeltaValue;
 };
 
 
+//----------------------------------------------------------------------------
+// qtractorTempoSpinBox -- A tempo/time-signature formatted spin-box widget.
+
+class qtractorTempoSpinBox : public QAbstractSpinBox
+{
+    Q_OBJECT
+
+public:
+
+	// Constructor.
+	qtractorTempoSpinBox(QWidget *pParent = 0);
+	// Destructor.
+	~qtractorTempoSpinBox();
+
+	// Nominal tempo value (BPM) accessors.
+	void setTempo(float fTempo, bool bNotifyChange = true);
+	float tempo() const;
+
+	// Nominal time-signature numerator (beats/bar) accessors.
+	void setBeatsPerBar(unsigned short iBeatsPerBar, bool bNotifyChange = true);
+	unsigned short beatsPerBar() const;
+
+	// Nominal time-signature denominator (beat-divisor) accessors.
+	void setBeatDivisor(unsigned short iBeatDivisor, bool bNotifyChange = true);
+	unsigned short beatDivisor() const;
+
+signals:
+
+	// Common value change notification.
+	void valueChanged(float, unsigned short, unsigned short);
+	void valueChanged(const QString&);
+
+protected:
+
+	// Mark that we got actual value.
+	void showEvent(QShowEvent *);
+
+	// Inherited/override methods.
+	QValidator::State validate(QString& sText,int& iPos) const;
+	void fixup(QString& sText) const;
+	void stepBy(int iSteps);
+	StepEnabled stepEnabled() const;
+
+	// Value/text format converters.
+	float tempoFromText(const QString& sText) const;
+	unsigned short beatsPerBarFromText(const QString& sText) const;
+	unsigned short beatDivisorFromText(const QString& sText) const;
+
+	QString textFromValue(float fTempo,
+		unsigned short iBeatsPerBar, unsigned short iBeatDivisor) const;
+
+protected slots:
+
+	// Pseudo-fixup slot.
+	void editingFinishedSlot();
+	void valueChangedSlot(const QString&);
+
+private:
+
+	// Instance variables.
+	float          m_fTempo;
+	unsigned short m_iBeatsPerBar;
+	unsigned short m_iBeatDivisor;
+};
+
+
 #endif  // __qtractorSpinBox_h
 
 
