@@ -486,10 +486,11 @@ void qtractorTrackForm::updateInstruments (void)
 	if (pInstruments == NULL)
 		return;
 
-	// Avoid superfluos change notifications...
+	// Avoid superfluous change notifications...
 	m_iDirtySetup++;
 
 	m_ui.InstrumentComboBox->clear();
+	m_ui.InstrumentComboBox->addItem(tr("(No instrument)"));
 	const QIcon& icon = QIcon(":/icons/itemInstrument.png");
 
 	// Take care of MIDI plugin instrument names...
@@ -607,6 +608,9 @@ void qtractorTrackForm::updateChannel ( int iChannel,
 
 	// MIDI channel patch...
 	const qtractorMidiBus::Patch& patch = m_pMidiBus->patch(iChannel);
+	QString sInstrumentName = patch.instrumentName;
+	if (sInstrumentName.isEmpty())
+		sInstrumentName = m_pMidiBus->instrumentName();
 	if (iBankSelMethod < 0)
 		iBankSelMethod = patch.bankSelMethod;
 #if 0
@@ -617,11 +621,16 @@ void qtractorTrackForm::updateChannel ( int iChannel,
 #endif
 
 	// Select instrument...
-	m_ui.InstrumentComboBox->setCurrentIndex(
-		m_ui.InstrumentComboBox->findText(patch.instrumentName));
+	int iInstrumentIndex
+		= m_ui.InstrumentComboBox->findText(sInstrumentName);
+	if (iInstrumentIndex < 0) {
+		iInstrumentIndex = 0;
+		sInstrumentName.clear();
+	}
+	m_ui.InstrumentComboBox->setCurrentIndex(iInstrumentIndex);
 
 	// Go and update the bank and program listings...
-	updateBanks(m_ui.InstrumentComboBox->currentText(),
+	updateBanks(sInstrumentName,
 		iBankSelMethod, iBank, iProg);
 
 	// Done.
@@ -1050,8 +1059,10 @@ void qtractorTrackForm::outputBusNameChanged ( const QString& sBusName )
 	}
 
 	// Recache the applicable MIDI output bus ...
-	if (trackType == qtractorTrack::Midi)
+	if (trackType == qtractorTrack::Midi) {
 		m_pMidiBus = midiBus();
+		updateInstruments();
+	}
 
 	channelChanged(m_ui.ChannelSpinBox->value());
 }
@@ -1151,8 +1162,13 @@ void qtractorTrackForm::bankChanged (void)
 	if (m_iDirtySetup > 0)
 		return;
 
-	updatePrograms(m_ui.InstrumentComboBox->currentText(),
-		midiBank(), midiProgram());
+	QString sInstrumentName;
+	if (m_ui.InstrumentComboBox->currentIndex() > 0)
+		sInstrumentName = m_ui.InstrumentComboBox->currentText();
+
+	updatePrograms(sInstrumentName,
+		midiBank(),
+		midiProgram());
 
 	progChanged();
 }
@@ -1168,7 +1184,9 @@ void qtractorTrackForm::progChanged (void)
 	if (m_pMidiBus) {
 		// Patch parameters...
 		unsigned short iChannel = m_ui.ChannelSpinBox->value() - 1;
-		const QString& sInstrumentName = m_ui.InstrumentComboBox->currentText();
+		QString sInstrumentName;
+		if (m_ui.InstrumentComboBox->currentIndex() > 0)
+			sInstrumentName = m_ui.InstrumentComboBox->currentText();
 		int iBankSelMethod = m_ui.BankSelMethodComboBox->currentIndex();
 		int iBank = midiBank();
 		int iProg = midiProgram();
