@@ -1,7 +1,7 @@
 // qtractorVstPlugin.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -30,12 +30,23 @@
 #include "qtractorMidiBuffer.h"
 
 #include "qtractorSession.h"
+#include "qtractorOptions.h"
 
 #include <QApplication>
 #include <QFileDialog>
 #include <QFileInfo>
 #include <QWidget>
 
+#if QT_VERSION < 0x040500
+namespace Qt {
+const WindowFlags WindowCloseButtonHint = WindowFlags(0x08000000);
+#if QT_VERSION < 0x040200
+const WindowFlags CustomizeWindowHint   = WindowFlags(0x02000000);
+#endif
+}
+#endif
+
+
 #if defined(Q_WS_X11)
 #include <QX11Info>
 #include <X11/Xlib.h>
@@ -740,7 +751,7 @@ void qtractorVstPlugin::selectProgram ( int iBank, int iProg )
 
 	// HACK: We don't change program-preset when
 	// we're supposed to be multi-timbral...
-	if (list()->flags() == qtractorPluginList::MidiBus)
+	if (list()->flags() & qtractorPluginList::MidiBus)
 		return;
 
 #ifdef CONFIG_DEBUG
@@ -890,16 +901,14 @@ void qtractorVstPlugin::openEditor ( QWidget */*pParent*/ )
 
 	// Create the new parent frame...
 	Qt::WindowFlags wflags = Qt::Window
-	#if QT_VERSION >= 0x040200
 		| Qt::CustomizeWindowHint
-	#if QT_VERSION >= 0x040500
-		| Qt::WindowCloseButtonHint
-	#endif
-	#endif
 		| Qt::WindowTitleHint
 		| Qt::WindowSystemMenuHint
 		| Qt::WindowMinMaxButtonsHint
-		| Qt::Tool;
+		| Qt::WindowCloseButtonHint;
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions && pOptions->bKeepToolsOnTop)
+		wflags |= Qt::Tool;
 
 	m_pEditorWidget = new EditorWidget(NULL, wflags);
 	m_pEditorWidget->open(this);
